@@ -3,16 +3,224 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: osddemo.c 2.3 2011/02/26 12:08:13 kls Exp $
+ * $Id: osddemo.c 5.1 2025/03/02 11:03:35 kls Exp $
  */
 
 #include <vdr/osd.h>
 #include <vdr/plugin.h>
 
-static const char *VERSION        = "0.2.0";
+static const char *VERSION        = "2.4.1";
 static const char *DESCRIPTION    = "Demo of arbitrary OSD setup";
 static const char *MAINMENUENTRY  = "Osd Demo";
 
+// --- DrawEllipses ----------------------------------------------------------
+
+void DrawEllipse(cOsd *Osd, int x1, int y1, int x2, int y2, int Quadrants)
+{
+  Osd->DrawRectangle(x1 + 2, y1 + 2, x2 - 2, y2 - 2, clrGreen);
+  Osd->DrawEllipse(x1 + 3, y1 + 3, x2 - 3, y2 - 3, clrRed, Quadrants);
+}
+
+void DrawEllipses(cOsd *Osd)
+{
+  int xa = 0;
+  int ya = 0;
+  int xb = Osd->Width() - 1;
+  int yb = Osd->Height() - 1;
+  int x0 = xa;
+  int x5 = xb;
+  int x1 = x0 + (xb - xa) / 5;
+  int x2 = x0 + (xb - xa) * 2 / 5;
+  int x3 = x0 + (xb - xa) * 3 / 5;
+  int x4 = x0 + (xb - xa) * 4 / 5;
+  int y0 = ya;
+  int y4 = yb;
+  int y2 = (y0 + y4) / 2;
+  int y1 = (y0 + y2) / 2;
+  int y3 = (y2 + y4) / 2;
+  Osd->DrawRectangle(xa, ya, xb, yb, clrGray50);
+  DrawEllipse(Osd, x4, y0, x5, y4, 0);
+  DrawEllipse(Osd, x2, y1, x3, y2, 1);
+  DrawEllipse(Osd, x1, y1, x2, y2, 2);
+  DrawEllipse(Osd, x1, y2, x2, y3, 3);
+  DrawEllipse(Osd, x2, y2, x3, y3, 4);
+  DrawEllipse(Osd, x3, y1, x4, y3, 5);
+  DrawEllipse(Osd, x1, y0, x3, y1, 6);
+  DrawEllipse(Osd, x0, y1, x1, y3, 7);
+  DrawEllipse(Osd, x1, y3, x3, y4, 8);
+  DrawEllipse(Osd, x3, y0, x4, y1, -1);
+  DrawEllipse(Osd, x0, y0, x1, y1, -2);
+  DrawEllipse(Osd, x0, y3, x1, y4, -3);
+  DrawEllipse(Osd, x3, y3, x4, y4, -4);
+  Osd->Flush();
+}
+
+// --- DrawSlopes ------------------------------------------------------------
+
+void DrawSlope(cOsd *Osd, int x1, int y1, int x2, int y2, int Type)
+{
+  Osd->DrawRectangle(x1 + 2, y1 + 2, x2 - 2, y2 - 2, clrGreen);
+  Osd->DrawSlope(x1 + 3, y1 + 3, x2 - 3, y2 - 3, clrRed, Type);
+}
+
+void DrawSlopes(cOsd *Osd)
+{
+  int xa = 0;
+  int ya = 0;
+  int xb = Osd->Width() - 1;
+  int yb = Osd->Height() - 1;
+  int x0 = xa;
+  int x4 = xb;
+  int x2 = (x0 + x4) / 2;
+  int x1 = (x0 + x2) / 2;
+  int x3 = (x2 + x4) / 2;
+  int y0 = ya;
+  int y3 = yb;
+  int y2 = (y0 + y3) / 2;
+  int y1 = (y0 + y2) / 2;
+  Osd->DrawRectangle(xa, ya, xb, yb, clrGray50);
+  DrawSlope(Osd, x0, y0, x2, y1, 0);
+  DrawSlope(Osd, x2, y0, x4, y1, 1);
+  DrawSlope(Osd, x0, y1, x2, y2, 2);
+  DrawSlope(Osd, x2, y1, x4, y2, 3);
+  DrawSlope(Osd, x0, y2, x1, y3, 4);
+  DrawSlope(Osd, x1, y2, x2, y3, 5);
+  DrawSlope(Osd, x2, y2, x3, y3, 6);
+  DrawSlope(Osd, x3, y2, x4, y3, 7);
+  Osd->Flush();
+}
+
+// --- DrawImages ------------------------------------------------------------
+
+struct tOsdImageRef {
+  int image;
+  cSize size;
+  };
+
+#define NUMOSDIMAGES 16
+#define NUMOSDIMAGEVARIANTS 8
+
+void DrawImages(cOsd *Osd)
+{
+  // Create images:
+  cImage *images[NUMOSDIMAGEVARIANTS];
+  for (int i = 0; i < NUMOSDIMAGEVARIANTS; i++) {
+      images[i] = new cImage(cSize(
+        i == 0 || i == 1 ? Osd->MaxPixmapSize().Width() + 1 : rand() % Osd->Width(),
+        i == 0 || i == 2 ? Osd->MaxPixmapSize().Height() + 1 : rand() % Osd->Height()));
+      for (int x = 0; x < images[i]->Width(); x++) {
+          for (int y = 0; y < images[i]->Height(); y++) {
+              images[i]->SetPixel(cPoint(x, y),
+                (!x || !y || x == images[i]->Width() - 1 || y == images[i]->Height() - 1) ? clrWhite :
+                (x > images[i]->Width() / 2 ?
+                  (y > images[i]->Height() / 2 ? clrBlue : clrGreen) :
+                  (y > images[i]->Height() / 2 ? clrRed : clrYellow)));
+              }
+          }
+      }
+  // Store images:
+  tOsdImageRef osdImages[NUMOSDIMAGES];
+  for (int i = 0; i < NUMOSDIMAGES; i++) {
+      osdImages[i].image = cOsdProvider::StoreImage(*images[i % NUMOSDIMAGEVARIANTS]);
+      osdImages[i].size.Set(images[i % NUMOSDIMAGEVARIANTS]->Size());
+      }
+  // Delete images:
+  for (int i = 0; i < NUMOSDIMAGEVARIANTS; i++)
+      delete images[i];
+  // Draw images:
+  for (int i = 0; i < NUMOSDIMAGES; i++)
+      Osd->DrawImage(cPoint(rand() % (Osd->Width() + osdImages[i].size.Width()), rand() % (Osd->Height() + osdImages[i].size.Height())).Shifted(-osdImages[i].size.Width(), -osdImages[i].size.Height()), osdImages[i].image);
+  // Drop image references:
+  for (int i = 0; i < NUMOSDIMAGES; i++)
+      cOsdProvider::DropImage(osdImages[i].image);
+  Osd->Flush();
+}
+
+// --- DrawEllipseAlignments -------------------------------------------------
+
+void DrawEllipseAlignments(cOsd *Osd)
+{
+  cFont *Font = cFont::CreateFont(Setup.FontOsd, 20);
+  int xa = 0;
+  int ya = 0;
+  int xb = Osd->Width() - 1;
+  int yb = Osd->Height() - 1;
+  Osd->DrawRectangle(xa, ya, xb, yb, clrBlack);
+  int d = 50;
+  int a = d / 2 + 1;
+  int t = a * 2;
+  int n = 30;
+  for (int i = 0; i < n; i++) {
+      Osd->DrawRectangle(a, t, a + d - 1, t + d - 1, clrGreen);
+      Osd->DrawEllipse(a, t - d / 2, a + d - 1, t, clrGreen, 6);
+      Osd->DrawEllipse(a, t + d, a + d - 1, t + d + d / 2, clrGreen, 8);
+      Osd->DrawText(a + d / 3, t + d / 3, itoa(d), clrRed, clrGreen, Font);
+      a += d + 5;
+      d++;
+      }
+  d = 50;
+  a = d * 3;
+  n = 20;
+  for (int i = 0; i < n; i++) {
+      Osd->DrawRectangle(t, a, t + d - 1, a + d - 1, clrGreen);
+      Osd->DrawEllipse(t - d / 2, a, t, a + d - 1, clrGreen, 7);
+      Osd->DrawEllipse(t + d, a, t + d + d / 2, a + d - 1, clrGreen, 5);
+      Osd->DrawText(t + d / 3, a + d / 3, itoa(d), clrRed, clrGreen, Font);
+      a += d + 5;
+      d++;
+      }
+  d = 50;
+  a = d * 3;
+  t = d * 5;
+  n = 30;
+  for (int i = 0; i < n; i++) {
+      Osd->DrawRectangle(a, t, a + d - 1, t + d - 1, clrGreen);
+      Osd->DrawEllipse(a, t - d, a + d - 1, t, clrGreen, 2);
+      Osd->DrawEllipse(a, t + d, a + d - 1, t + d + d, clrGreen, 3);
+      Osd->DrawText(a + d / 3, t + d / 3, itoa(d), clrRed, clrGreen, Font);
+      a += d + 5;
+      d++;
+      }
+  d = 50;
+  a = d * 3;
+  t = d * 9;
+  n = 30;
+  for (int i = 0; i < n; i++) {
+      Osd->DrawRectangle(a, t, a + d - 1, t + d - 1, clrGreen);
+      Osd->DrawEllipse(a, t - d, a + d - 1, t, clrGreen, 1);
+      Osd->DrawEllipse(a, t + d, a + d - 1, t + d + d, clrGreen, 4);
+      Osd->DrawText(a + d / 3, t + d / 3, itoa(d), clrRed, clrGreen, Font);
+      a += d + 5;
+      d++;
+      }
+  d = 50;
+  a = d * 12;
+  t = d * 5;
+  n = 20;
+  for (int i = 0; i < n; i++) {
+      Osd->DrawRectangle(t, a, t + d - 1, a + d - 1, clrGreen);
+      Osd->DrawEllipse(t - d, a, t, a + d - 1, clrGreen, 2);
+      Osd->DrawEllipse(t + d, a, t + d + d, a + d - 1, clrGreen, 1);
+      Osd->DrawText(t + d / 3, a + d / 3, itoa(d), clrRed, clrGreen, Font);
+      a += d + 5;
+      d++;
+      }
+  d = 50;
+  a = d * 12;
+  t = d * 9;
+  n = 20;
+  for (int i = 0; i < n; i++) {
+      Osd->DrawRectangle(t, a, t + d - 1, a + d - 1, clrGreen);
+      Osd->DrawEllipse(t - d, a, t, a + d - 1, clrGreen, 3);
+      Osd->DrawEllipse(t + d, a, t + d + d, a + d - 1, clrGreen, 4);
+      Osd->DrawText(t + d / 3, a + d / 3, itoa(d), clrRed, clrGreen, Font);
+      a += d + 5;
+      d++;
+      }
+  Osd->Flush();
+  delete Font;
+}
+
 // --- cLineGame -------------------------------------------------------------
 
 class cLineGame : public cOsdObject {
@@ -23,15 +231,15 @@ private:
   tColor color;
 public:
   cLineGame(void);
-  virtual ~cLineGame();
-  virtual void Show(void);
-  virtual eOSState ProcessKey(eKeys Key);
+  virtual ~cLineGame() override;
+  virtual void Show(void) override;
+  virtual eOSState ProcessKey(eKeys Key) override;
   };
 
 cLineGame::cLineGame(void)
 {
   osd = NULL;
-  x = y = 50;
+  x = y = 0;
   color = clrRed;
 }
 
@@ -42,12 +250,23 @@ cLineGame::~cLineGame()
 
 void cLineGame::Show(void)
 {
-  osd = cOsdProvider::NewOsd(100, 50, 50);
+  osd = cOsdProvider::NewOsd(cOsd::OsdLeft(), cOsd::OsdTop());
   if (osd) {
-     tArea Area = { 0, 0, 99, 199,  4 };
-     osd->SetAreas(&Area, 1);
-     osd->DrawRectangle(0, 0, 99, 199, clrGray50);
-     osd->Flush();
+     int x1 = cOsd::OsdWidth() - 1;
+     int y1 = cOsd::OsdHeight() - 1;
+     while (x1 > 0 && y1 > 0) {
+           tArea Area = { 0, 0, x1, y1, 4 };
+           if (osd->CanHandleAreas(&Area, 1) == oeOk) {
+              osd->SetAreas(&Area, 1);
+              osd->DrawRectangle(0, 0, osd->Width() - 1, osd->Height() - 1, clrGray50);
+              osd->Flush();
+              x = osd->Width() / 2;
+              y = osd->Height() / 2;
+              break;
+              }
+           x1 = x1 * 9 / 10;
+           y1 = y1 * 9 / 10;
+           }
      }
 }
 
@@ -55,19 +274,25 @@ eOSState cLineGame::ProcessKey(eKeys Key)
 {
   eOSState state = cOsdObject::ProcessKey(Key);
   if (state == osUnknown) {
+     const int d = 4;
      switch (Key & ~k_Repeat) {
-       case kUp:     if (y > 0)   y--; break;
-       case kDown:   if (y < 196) y++; break;
-       case kLeft:   if (x > 0)   x--; break;
-       case kRight:  if (x < 96)  x++; break;
+       case kUp:     y = max(0, y - d); break;
+       case kDown:   y = min(osd->Height() - d, y + d); break;
+       case kLeft:   x = max(0, x - d); break;
+       case kRight:  x = min(osd->Width() - d, x + d); break;
        case kRed:    color = clrRed; break;
        case kGreen:  color = clrGreen; break;
        case kYellow: color = clrYellow; break;
        case kBlue:   color = clrBlue; break;
+       case k1:      DrawEllipses(osd);
+                     return osContinue;
+       case k2:      DrawSlopes(osd);
+                     return osContinue;
+       case kBack:
        case kOk:     return osEnd;
        default: return state;
        }
-     osd->DrawRectangle(x, y, x + 3, y + 3, color);
+     osd->DrawRectangle(x, y, x + d - 1, y + d - 1, color);
      osd->Flush();
      state = osContinue;
      }
@@ -84,13 +309,14 @@ private:
   bool clockwise;
   cPixmap *destroyablePixmap;
   cPixmap *toggleablePixmap;
-  virtual void Action(void);
-  cPixmap *CreateTextPixmap(const char *s, int Line, int Layer, tColor ColorFg, tColor ColorBg, const cFont *Font = NULL);
+  bool SetArea(void);
+  virtual void Action(void) override;
+  cPixmap *CreateTextPixmap(const char *s, int Line, int Layer, tColor ColorFg, tColor ColorBg, const cFont *Font);
 public:
   cTrueColorDemo(void);
-  virtual ~cTrueColorDemo();
-  virtual void Show(void);
-  virtual eOSState ProcessKey(eKeys Key);
+  virtual ~cTrueColorDemo() override;
+  virtual void Show(void) override;
+  virtual eOSState ProcessKey(eKeys Key) override;
   };
 
 cTrueColorDemo::cTrueColorDemo(void)
@@ -103,20 +329,19 @@ cTrueColorDemo::cTrueColorDemo(void)
 
 cTrueColorDemo::~cTrueColorDemo()
 {
+  Cancel(3);
   delete osd;
 }
 
 cPixmap *cTrueColorDemo::CreateTextPixmap(const char *s, int Line, int Layer, tColor ColorFg, tColor ColorBg, const cFont *Font)
 {
-  if (!Font)
-     Font = cFont::GetFont(fontOsd);
   const int h = Font->Height(s);
   int w = Font->Width(s);
   cPixmap *Pixmap = osd->CreatePixmap(Layer, cRect((osd->Width() - w) / 2, Line, w, h));
   if (Pixmap) {
      Pixmap->Clear();
      Pixmap->SetAlpha(0);
-     Pixmap->DrawText(cPoint(0, 0), s, ColorFg, ColorBg, Font);
+     Pixmap->DrawText(cPoint(0, 0), s, ColorFg, ColorBg, Font, w);
      }
   return Pixmap;
 }
@@ -130,6 +355,9 @@ void cTrueColorDemo::Action(void)
   cPixmap *TilePixmap = NULL;
   cPixmap *ScrollPixmap = NULL;
   cPixmap *AnimPixmap = NULL;
+  cFont *OsdFont = cFont::CreateFont(Setup.FontOsd, Setup.FontOsdSize);
+  cFont *SmlFont = cFont::CreateFont(Setup.FontSml, Setup.FontSmlSize);
+  cFont *LrgFont = cFont::CreateFont(Setup.FontOsd, osd->Height() / 10);
   int FrameTime = 40; // ms
   int FadeTime = 1000; // ms
   int MoveTime = 4000; // ms
@@ -201,10 +429,9 @@ void cTrueColorDemo::Action(void)
                        cRect r = ScrollPixmap->DrawPort();
                        r.SetPoint(-r.X(), -r.Y());
                        ScrollPixmap->Pan(cPoint(0, 0), r);
-                       const cFont *Font = cFont::GetFont(fontOsd);
                        cString s = cString::sprintf("Line %d", ++ScrollLineNumber);
                        ScrollPixmap->DrawRectangle(cRect(0, ScrollPixmap->ViewPort().Height(), ScrollPixmap->DrawPort().Width(), ScrollPixmap->DrawPort().Height()), clrTransparent);
-                       ScrollPixmap->DrawText(cPoint(0, ScrollPixmap->ViewPort().Height()), s, clrYellow, clrTransparent, Font);
+                       ScrollPixmap->DrawText(cPoint(0, ScrollPixmap->ViewPort().Height()), s, clrYellow, clrTransparent, OsdFont);
                        ScrollStartTime = Now;
                        }
                     else {
@@ -233,18 +460,15 @@ void cTrueColorDemo::Action(void)
         if (!Animated) {
            switch (State) {
              case 0: {
-                       if (cFont *Font = cFont::CreateFont(DefaultFontOsd, osd->Height() / 10)) {
-                          FadeInPixmap = CreateTextPixmap("VDR", Line, 1, clrYellow, clrTransparent, Font);
-                          if (FadeInPixmap)
-                             Line += FadeInPixmap->DrawPort().Height();
-                          delete Font;
-                          Start = cTimeMs::Now();
-                          }
+                       FadeInPixmap = CreateTextPixmap("VDR", Line, 1, clrYellow, clrTransparent, LrgFont);
+                       if (FadeInPixmap)
+                          Line += FadeInPixmap->DrawPort().Height();
+                       Start = cTimeMs::Now();
                        State++;
                      }
                      break;
-             case 1: { 
-                       FadeInPixmap = CreateTextPixmap("Video Disk Recorder", Line, 3, clrYellow, clrTransparent);
+             case 1: {
+                       FadeInPixmap = CreateTextPixmap("Video Disk Recorder", Line, 3, clrYellow, clrTransparent, OsdFont);
                        if (FadeInPixmap)
                           Line += FadeInPixmap->DrawPort().Height();
                        Start = cTimeMs::Now();
@@ -252,7 +476,7 @@ void cTrueColorDemo::Action(void)
                      }
                      break;
              case 2: {
-                       FadeInPixmap = CreateTextPixmap("True Color OSD Demo", Line, 1, clrYellow, clrTransparent);
+                       FadeInPixmap = CreateTextPixmap("True Color OSD Demo", Line, 1, clrYellow, clrTransparent, OsdFont);
                        if (FadeInPixmap)
                           Line += FadeInPixmap->DrawPort().Height();
                        Start = cTimeMs::Now();
@@ -260,11 +484,9 @@ void cTrueColorDemo::Action(void)
                      }
                      break;
              case 3: {
-                       if (cFont *Font = cFont::CreateFont(DefaultFontOsd, osd->Height() / 10)) {
-                          NextPixmap = CreateTextPixmap("Millions of colors", Line, 1, clrYellow, clrTransparent, Font);
-                          if (NextPixmap) {
-                             FadeInPixmap = NextPixmap;
-                             }
+                       NextPixmap = CreateTextPixmap("Millions of colors", Line, 1, clrYellow, clrTransparent, LrgFont);
+                       if (NextPixmap) {
+                          FadeInPixmap = NextPixmap;
                           Start = cTimeMs::Now();
                           StartLine = Line;
                           Line += NextPixmap->DrawPort().Height();
@@ -299,7 +521,7 @@ void cTrueColorDemo::Action(void)
                      }
                      break;
              case 6: {
-                       TilePixmap = CreateTextPixmap("Tiled Pixmaps", StartLine, 1, clrRed, clrWhite);
+                       TilePixmap = CreateTextPixmap("Tiled Pixmaps", StartLine, 1, clrRed, clrWhite, OsdFont);
                        if (TilePixmap) {
                           TilePixmap->SetViewPort(TilePixmap->ViewPort().Grown(TilePixmap->DrawPort().Width(), TilePixmap->DrawPort().Height()));
                           TilePixmap->SetAlpha(200);
@@ -315,15 +537,14 @@ void cTrueColorDemo::Action(void)
                      }
                      break;
              case 7: {
-                       const cFont *Font = cFont::GetFont(fontOsd);
                        const char *Text = "Scrolling Pixmaps";
-                       int w = Font->Width(Text);
-                       int h = Font->Height();
+                       int w = OsdFont->Width(Text);
+                       int h = OsdFont->Height();
                        if (cPixmap *Pixmap = osd->CreatePixmap(2, cRect((osd->Width() - w) / 2, StartLine, w, 2 * h), cRect(0, 0, w, 3 * h))) {
                           Pixmap->Clear();
-                          Pixmap->DrawText(cPoint(0, 0), Text, clrYellow, clrTransparent, Font);
+                          Pixmap->DrawText(cPoint(0, 0), Text, clrYellow, clrTransparent, OsdFont);
                           cString s = cString::sprintf("Line %d", ++ScrollLineNumber);
-                          Pixmap->DrawText(cPoint(0, Pixmap->ViewPort().Height()), s, clrYellow, clrTransparent, Font);
+                          Pixmap->DrawText(cPoint(0, Pixmap->ViewPort().Height()), s, clrYellow, clrTransparent, OsdFont);
                           ScrollPixmap = Pixmap;
                           ScrollStart.Set(0, 0);
                           ScrollEnd.Set(0, -h);
@@ -333,21 +554,24 @@ void cTrueColorDemo::Action(void)
                      }
                      break;
              case 8: {
-                       const cFont *Font = cFont::GetFont(fontSml);
                        const char *Text = "Animation";
-                       const int Size = Font->Width(Text) + 10;
+                       const int Size = SmlFont->Width(Text) + 10;
                        const int NumDots = 12;
                        const int AnimFrames = NumDots;
+                       int Rows = min(osd->MaxPixmapSize().Height() / Size, AnimFrames);
+                       int Cols = (AnimFrames + Rows - 1) / Rows;
                        // Temporarily using pixmap layer 0 to have the text alpha blended:
-                       AnimPixmap = osd->CreatePixmap(0, cRect((osd->Width() - Size) / 2, StartLine, Size, Size), cRect(0, 0, Size, Size * AnimFrames));
+                       AnimPixmap = osd->CreatePixmap(0, cRect((osd->Width() - Size) / 2, StartLine, Size, Size), cRect(0, 0, Size * Cols, Size * Rows));
                        if (AnimPixmap) {
                           AnimPixmap->SetAlpha(0);
                           AnimPixmap->Clear();
                           const int Diameter = Size / 5;
-                          int xc = Size / 2 - Diameter / 2;
                           for (int Frame = 0; Frame < AnimFrames; Frame++) {
-                              AnimPixmap->DrawEllipse(cRect(0, Frame * Size, Size, Size), 0xDDFFFFFF);
-                              int yc = Frame * Size + Size / 2 - Diameter / 2;
+                              int x0 = Frame / Rows * Size;
+                              int y0 = Frame % Rows * Size;
+                              AnimPixmap->DrawEllipse(cRect(x0, y0, Size, Size), 0xDDFFFFFF);
+                              int xc = x0 + Size / 2 - Diameter / 2;
+                              int yc = y0 + Size / 2 - Diameter / 2;
                               int Color = 0xFF;
                               int Delta = Color / NumDots / 3;
                               for (int a = 0; a < NumDots; a++) {
@@ -357,7 +581,7 @@ void cTrueColorDemo::Action(void)
                                   AnimPixmap->DrawEllipse(cRect(x, y, Diameter, Diameter), ArgbToColor(0xFF, Color, Color, Color));
                                   Color -= Delta;
                                   }
-                              AnimPixmap->DrawText(cPoint(0, Frame * Size), Text, clrBlack, clrTransparent, cFont::GetFont(fontSml), Size, Size, taCenter);
+                              AnimPixmap->DrawText(cPoint(x0, y0), Text, clrBlack, clrTransparent, SmlFont, Size, Size, taCenter);
                               }
                           AnimPixmap->SetLayer(3); // now setting the actual pixmap layer
                           FadeInPixmap = AnimPixmap;
@@ -391,15 +615,28 @@ void cTrueColorDemo::Action(void)
         if (Delta < FrameTime)
            cCondWait::SleepMs(FrameTime - Delta);
         }
+  destroyablePixmap = NULL;
+  toggleablePixmap = NULL;
+  delete OsdFont;
+  delete SmlFont;
+  delete LrgFont;
+}
+
+bool cTrueColorDemo::SetArea(void)
+{
+  if (osd) {
+     tArea Area = { 0, 0, cOsd::OsdWidth() - 1, cOsd::OsdHeight() - 1,  32 };
+     return osd->SetAreas(&Area, 1) == oeOk;
+     }
+  return false;
 }
 
 void cTrueColorDemo::Show(void)
 {
-  osd = cOsdProvider::NewOsd(cOsd::OsdLeft(), cOsd::OsdTop(), 50);
+  osd = cOsdProvider::NewOsd(cOsd::OsdLeft(), cOsd::OsdTop());
   if (osd) {
-     tArea Area = { 0, 0, cOsd::OsdWidth() - 1, cOsd::OsdHeight() - 1,  32 };
-     if (osd->SetAreas(&Area, 1) == oeOk) {
-        osd->DrawRectangle(0, 0, osd->Width() -1 , osd->Height() - 1, clrGray50);
+     if (SetArea()) {
+        osd->DrawRectangle(0, 0, osd->Width() - 1, osd->Height() - 1, clrGray50);
         osd->Flush();
         Start();
         }
@@ -426,6 +663,23 @@ eOSState cTrueColorDemo::ProcessKey(eKeys Key)
        case kGreen:  if (toggleablePixmap)
                         toggleablePixmap->SetLayer(-toggleablePixmap->Layer());
                      break;
+       case k1:      Cancel(3);
+                     SetArea();
+                     DrawEllipses(osd);
+                     break;
+       case k2:      Cancel(3);
+                     SetArea();
+                     DrawSlopes(osd);
+                     break;
+       case k3:      Cancel(3);
+                     SetArea();
+                     DrawImages(osd);
+                     break;
+       case k4:      Cancel(3);
+                     SetArea();
+                     DrawEllipseAlignments(osd);
+                     return osContinue;
+       case kBack:
        case kOk:     return osEnd;
        default: return state;
        }
@@ -441,17 +695,17 @@ private:
   // Add any member variables or functions you may need here.
 public:
   cPluginOsddemo(void);
-  virtual ~cPluginOsddemo();
-  virtual const char *Version(void) { return VERSION; }
-  virtual const char *Description(void) { return DESCRIPTION; }
-  virtual const char *CommandLineHelp(void);
-  virtual bool ProcessArgs(int argc, char *argv[]);
-  virtual bool Start(void);
-  virtual void Housekeeping(void);
-  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
-  virtual cOsdObject *MainMenuAction(void);
-  virtual cMenuSetupPage *SetupMenu(void);
-  virtual bool SetupParse(const char *Name, const char *Value);
+  virtual ~cPluginOsddemo() override;
+  virtual const char *Version(void) override { return VERSION; }
+  virtual const char *Description(void) override { return DESCRIPTION; }
+  virtual const char *CommandLineHelp(void) override;
+  virtual bool ProcessArgs(int argc, char *argv[]) override;
+  virtual bool Start(void) override;
+  virtual void Housekeeping(void) override;
+  virtual const char *MainMenuEntry(void) override { return MAINMENUENTRY; }
+  virtual cOsdObject *MainMenuAction(void) override;
+  virtual cMenuSetupPage *SetupMenu(void) override;
+  virtual bool SetupParse(const char *Name, const char *Value) override;
   };
 
 cPluginOsddemo::cPluginOsddemo(void)
