@@ -3,24 +3,24 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: svccli.c 1.2 2007/08/15 13:18:08 kls Exp $
+ * $Id: svccli.c 5.1 2025/03/02 11:03:35 kls Exp $
  */
 
 #include <stdlib.h>
 #include <vdr/interface.h>
 #include <vdr/plugin.h>
 
-static const char *VERSION        = "0.1.2";
+static const char *VERSION        = "2.4.0";
 static const char *DESCRIPTION    = "Service demo client";
 static const char *MAINMENUENTRY  = "Service demo";
 
 class cPluginSvcCli : public cPlugin {
 public:
-  virtual const char *Version(void) { return VERSION; }
-  virtual const char *Description(void) { return DESCRIPTION; }
-  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
-  virtual cOsdObject *MainMenuAction(void);
-  virtual bool Service(const char *Id, void *Data);
+  virtual const char *Version(void) override { return VERSION; }
+  virtual const char *Description(void) override { return DESCRIPTION; }
+  virtual const char *MainMenuEntry(void) override { return MAINMENUENTRY; }
+  virtual cOsdObject *MainMenuAction(void) override;
+  virtual bool Service(const char *Id, void *Data) override;
   };
 
 struct ReportBoredPlugin_v1_0 {
@@ -32,7 +32,7 @@ struct AddService_v1_0 {
   int sum;
   };
 
-// --- cPluginSvcCli ----------------------------------------------------------
+// --- cPluginSvcCli ---------------------------------------------------------
 
 cOsdObject *cPluginSvcCli::MainMenuAction(void)
 {
