@@ -3,21 +3,21 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: svcsvr.c 1.2 2007/08/15 13:18:59 kls Exp $
+ * $Id: svcsvr.c 5.1 2025/03/02 11:03:35 kls Exp $
  */
 
 #include <stdlib.h>
 #include <vdr/interface.h>
 #include <vdr/plugin.h>
 
-static const char *VERSION        = "0.1.2";
+static const char *VERSION        = "2.4.0";
 static const char *DESCRIPTION    = "Service demo server";
 
 class cPluginSvcSvr : public cPlugin {
 public:
-  virtual const char *Version(void) { return VERSION; }
-  virtual const char *Description(void) { return DESCRIPTION; }
-  virtual bool Service(const char *Id, void *Data);
+  virtual const char *Version(void) override { return VERSION; }
+  virtual const char *Description(void) override { return DESCRIPTION; }
+  virtual bool Service(const char *Id, void *Data) override;
   };
 
 struct ReportBoredPlugin_v1_0 {
@@ -29,7 +29,7 @@ struct AddService_v1_0 {
   int sum;
   };
 
-// --- cPluginSvcSvr ----------------------------------------------------------
+// --- cPluginSvcSvr ---------------------------------------------------------
 
 bool cPluginSvcSvr::Service(const char *Id, void *Data)
 {
