@@ -3,15 +3,16 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: skincurses.c 2.5 2011/01/04 08:52:03 kls Exp $
+ * $Id: skincurses.c 5.3 2025/04/12 20:40:48 kls Exp $
  */
 
 #include <ncurses.h>
 #include <vdr/osd.h>
 #include <vdr/plugin.h>
 #include <vdr/skins.h>
+#include <vdr/videodir.h>
 
-static const char *VERSION        = "0.1.9";
+static const char *VERSION        = "2.4.3";
 static const char *DESCRIPTION    = trNOOP("A text only skin");
 static const char *MAINMENUENTRY  = NULL;
 
@@ -19,14 +20,15 @@ static const char *MAINMENUENTRY  = NULL;
 
 class cCursesFont : public cFont {
 public:
-  virtual int Width(uint c) const { return 1; }
-  virtual int Width(const char *s) const { return s ? Utf8StrLen(s) : 0; }
-  virtual int Height(void) const { return 1; }
-  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const {}
-  virtual void DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const {}
+  virtual int Width(void) const override { return 1; }
+  virtual int Width(uint c) const override { return 1; }
+  virtual int Width(const char *s) const override { return s ? Utf8StrLen(s) : 0; }
+  virtual int Height(void) const override { return 1; }
+  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const override {}
+  virtual void DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const override {}
   };
 
-static const cCursesFont Font;
+static const cCursesFont Font = cCursesFont(); // w/o the '= cCursesFont()' gcc 4.6 complains - can anybody explain why this is necessary?
 
 // --- cCursesOsd ------------------------------------------------------------
 
@@ -58,18 +60,16 @@ static int ScOsdHeight = 20;
 class cCursesOsd : public cOsd {
 private:
   WINDOW *savedRegion;
-  WINDOW *window;
-  enum { MaxColorPairs = 16 };
-  int colorPairs[MaxColorPairs];
+  cVector<int> colorPairs;
   void SetColor(int colorFg, int colorBg = clrBackground);
 public:
   cCursesOsd(int Left, int Top);
-  virtual ~cCursesOsd();
-  virtual void SaveRegion(int x1, int y1, int x2, int y2);
-  virtual void RestoreRegion(void);
-  virtual void DrawText(int x, int y, const char *s, tColor ColorFg, tColor ColorBg, const cFont *Font, int Width = 0, int Height = 0, int Alignment = taDefault);
-  virtual void DrawRectangle(int x1, int y1, int x2, int y2, tColor Color);
-  virtual void Flush(void);
+  virtual ~cCursesOsd() override;
+  virtual void SaveRegion(int x1, int y1, int x2, int y2) override;
+  virtual void RestoreRegion(void) override;
+  virtual void DrawText(int x, int y, const char *s, tColor ColorFg, tColor ColorBg, const cFont *Font, int Width = 0, int Height = 0, int Alignment = taDefault) override;
+  virtual void DrawRectangle(int x1, int y1, int x2, int y2, tColor Color) override;
+  virtual void Flush(void) override;
   };
 
 cCursesOsd::cCursesOsd(int Left, int Top)
@@ -77,40 +77,34 @@ cCursesOsd::cCursesOsd(int Left, int Top)
 {
   savedRegion = NULL;
 
-  memset(colorPairs, 0x00, sizeof(colorPairs));
   start_color();
   leaveok(stdscr, true);
+  refresh(); // to react on changes to screen size
 
-  window = subwin(stdscr, ScOsdHeight, ScOsdWidth, 0, 0);
-  syncok(window, true);
+  int begy, begx;
+  int maxy, maxx;
+  getmaxyx(stdscr, maxy, maxx);
+  getbegyx(stdscr, begy, begx);
+  ScOsdWidth  = maxx - begx;
+  ScOsdHeight = maxy - begy;
 }
 
 cCursesOsd::~cCursesOsd()
 {
-  if (window) {
-     werase(window);
-     Flush();
-     delwin(window);
-     window = NULL;
-     }
+  erase();
+  Flush();
 }
 
 void cCursesOsd::SetColor(int colorFg, int colorBg)
 {
   int color = (colorBg << 16) | colorFg | 0x80000000;
-  for (int i = 0; i < MaxColorPairs; i++) {
-      if (!colorPairs[i]) {
-         colorPairs[i] = color;
-         init_pair(i + 1, colorFg, colorBg);
-         //XXX??? attron(COLOR_PAIR(WHITE_ON_BLUE));
-         wattrset(window, COLOR_PAIR(i + 1));
-         break;
-         }
-      else if (color == colorPairs[i]) {
-         wattrset(window, COLOR_PAIR(i + 1));
-         break;
-         }
-      }
+  int i = colorPairs.IndexOf(color);
+  if (i < 0) {
+     colorPairs.Append(color);
+     i = colorPairs.Size() - 1;
+     init_pair(i + 1, colorFg, colorBg);
+     }
+  attrset(COLOR_PAIR(i + 1));
 }
 
 void cCursesOsd::SaveRegion(int x1, int y1, int x2, int y2)
@@ -120,13 +114,14 @@ void cCursesOsd::SaveRegion(int x1, int y1, int x2, int y2)
      savedRegion = NULL;
      }
   savedRegion = newwin(y2 - y1 + 1, x2 - x1 + 1, y1, x1);
-  copywin(window, savedRegion, y1, x1, 0, 0, y2 - y1, x2 - x1, false);
+  if (savedRegion)
+     copywin(stdscr, savedRegion, y1, x1, 0, 0, y2 - y1, x2 - x1, false);
 }
 
 void cCursesOsd::RestoreRegion(void)
 {
   if (savedRegion) {
-     copywin(savedRegion, window, 0, 0, savedRegion->_begy, savedRegion->_begx, savedRegion->_maxy - savedRegion->_begy, savedRegion->_maxx - savedRegion->_begx, false);
+     overwrite(savedRegion, stdscr);
      delwin(savedRegion);
      savedRegion = NULL;
      }
@@ -166,18 +161,22 @@ void cCursesOsd::DrawText(int x, int y, const char *s, tColor ColorFg, tColor Co
         }
      }
   SetColor(ColorFg, ColorBg);
-  wmove(window, y, x); // ncurses wants 'y' before 'x'!
-  waddnstr(window, s, Width ? Width : ScOsdWidth - x);
+  mvaddnstr(y, x, s, Width ? Width : ScOsdWidth - x);
 }
 
 void cCursesOsd::DrawRectangle(int x1, int y1, int x2, int y2, tColor Color)
 {
   SetColor(Color, Color);
-  for (int y = y1; y <= y2; y++) {
-      wmove(window, y, x1); // ncurses wants 'y' before 'x'!
-      whline(window, ' ', x2 - x1 + 1);
-      }
-  wsyncup(window); // shouldn't be necessary because of 'syncok()', but w/o it doesn't work
+  int dx = x2 - x1;
+  int dy = y2 - y1;
+  if (dx >= dy) {
+     for (int y = y1; y <= y2; y++)
+         mvhline(y, x1, ' ', dx + 1);
+     }
+  else {
+     for (int x = x1; x <= x2; x++)
+         mvvline(y1, x, ' ', dy + 1);
+     }
 }
 
 void cCursesOsd::Flush(void)
@@ -194,11 +193,11 @@ private:
   bool message;
 public:
   cSkinCursesDisplayChannel(bool WithInfo);
-  virtual ~cSkinCursesDisplayChannel();
-  virtual void SetChannel(const cChannel *Channel, int Number);
-  virtual void SetEvents(const cEvent *Present, const cEvent *Following);
-  virtual void SetMessage(eMessageType Type, const char *Text);
-  virtual void Flush(void);
+  virtual ~cSkinCursesDisplayChannel() override;
+  virtual void SetChannel(const cChannel *Channel, int Number) override;
+  virtual void SetEvents(const cEvent *Present, const cEvent *Following) override;
+  virtual void SetMessage(eMessageType Type, const char *Text) override;
+  virtual void Flush(void) override;
   };
 
 cSkinCursesDisplayChannel::cSkinCursesDisplayChannel(bool WithInfo)
@@ -262,29 +261,33 @@ void cSkinCursesDisplayChannel::Flush(void)
 class cSkinCursesDisplayMenu : public cSkinDisplayMenu {
 private:
   cOsd *osd;
+  cString title;
+  int lastDiskUsageState;
+  void DrawTitle(void);
   void DrawScrollbar(int Total, int Offset, int Shown, int Top, int Height, bool CanScrollUp, bool CanScrollDown);
   void SetTextScrollbar(void);
 public:
   cSkinCursesDisplayMenu(void);
-  virtual ~cSkinCursesDisplayMenu();
-  virtual void Scroll(bool Up, bool Page);
-  virtual int MaxItems(void);
-  virtual void Clear(void);
-  virtual void SetTitle(const char *Title);
-  virtual void SetButtons(const char *Red, const char *Green = NULL, const char *Yellow = NULL, const char *Blue = NULL);
-  virtual void SetMessage(eMessageType Type, const char *Text);
-  virtual void SetItem(const char *Text, int Index, bool Current, bool Selectable);
-  virtual void SetScrollbar(int Total, int Offset);
-  virtual void SetEvent(const cEvent *Event);
-  virtual void SetRecording(const cRecording *Recording);
-  virtual void SetText(const char *Text, bool FixedFont);
-  virtual const cFont *GetTextAreaFont(bool FixedFont) const { return &Font; }
-  virtual void Flush(void);
+  virtual ~cSkinCursesDisplayMenu() override;
+  virtual void Scroll(bool Up, bool Page) override;
+  virtual int MaxItems(void) override;
+  virtual void Clear(void) override;
+  virtual void SetTitle(const char *Title) override;
+  virtual void SetButtons(const char *Red, const char *Green = NULL, const char *Yellow = NULL, const char *Blue = NULL) override;
+  virtual void SetMessage(eMessageType Type, const char *Text) override;
+  virtual void SetItem(const char *Text, int Index, bool Current, bool Selectable) override;
+  virtual void SetScrollbar(int Total, int Offset) override;
+  virtual void SetEvent(const cEvent *Event) override;
+  virtual void SetRecording(const cRecording *Recording) override;
+  virtual void SetText(const char *Text, bool FixedFont) override;
+  virtual const cFont *GetTextAreaFont(bool FixedFont) const override { return &Font; }
+  virtual void Flush(void) override;
   };
 
 cSkinCursesDisplayMenu::cSkinCursesDisplayMenu(void)
 {
   osd = new cCursesOsd(0, 0);
+  lastDiskUsageState = -1;
   osd->DrawRectangle(0, 0, ScOsdWidth - 1, ScOsdHeight - 1, clrBackground);
 }
 
@@ -332,9 +335,16 @@ void cSkinCursesDisplayMenu::Clear(void)
   textScroller.Reset();
 }
 
+void cSkinCursesDisplayMenu::DrawTitle(void)
+{
+  bool WithDisk = MenuCategory() == mcMain || MenuCategory() == mcRecording;
+  osd->DrawText(0, 0, WithDisk ? cString::sprintf("%s  -  %s", *title, *cVideoDiskUsage::String()) : title, clrBlack, clrCyan, &Font, ScOsdWidth);
+}
+
 void cSkinCursesDisplayMenu::SetTitle(const char *Title)
 {
-  osd->DrawText(0, 0, Title, clrBlack, clrCyan, &Font, ScOsdWidth);
+  title = Title;
+  DrawTitle();
 }
 
 void cSkinCursesDisplayMenu::SetButtons(const char *Red, const char *Green, const char *Yellow, const char *Blue)
@@ -375,13 +385,13 @@ void cSkinCursesDisplayMenu::SetItem(const char *Text, int Index, bool Current,
   for (int i = 0; i < MaxTabs; i++) {
       const char *s = GetTabbedText(Text, i);
       if (s) {
-         int xt = Tab(i) / 12;// Tab() is in "pixel" - see also skins.c!!!
+         int xt = Tab(i) / AvgCharWidth();// Tab() is in "pixel" - see also skins.c!!!
          osd->DrawText(xt, y, s, ColorFg, ColorBg, &Font, ScOsdWidth - 2 - xt);
          }
       if (!Tab(i + 1))
          break;
       }
-  SetEditableWidth(ScOsdWidth - 2 - Tab(1) / 12); // Tab() is in "pixel" - see also skins.c!!!
+  SetEditableWidth(ScOsdWidth - 2 - Tab(1) / AvgCharWidth()); // Tab() is in "pixel" - see also skins.c!!!
 }
 
 void cSkinCursesDisplayMenu::SetScrollbar(int Total, int Offset)
@@ -395,8 +405,7 @@ void cSkinCursesDisplayMenu::SetEvent(const cEvent *Event)
      return;
   int y = 2;
   cTextScroller ts;
-  char t[32];
-  snprintf(t, sizeof(t), "%s  %s - %s", *Event->GetDateString(), *Event->GetTimeString(), *Event->GetEndTimeString());
+  cString t = cString::sprintf("%s  %s - %s", *Event->GetDateString(), *Event->GetTimeString(), *Event->GetEndTimeString());
   ts.Set(osd, 0, y, ScOsdWidth, ScOsdHeight - y - 2, t, &Font, clrYellow, clrBackground);
   if (Event->Vps() && Event->Vps() != Event->StartTime()) {
      cString buffer = cString::sprintf(" VPS: %s", *Event->GetVpsString());
@@ -435,13 +444,21 @@ void cSkinCursesDisplayMenu::SetRecording(const cRecording *Recording)
   const cRecordingInfo *Info = Recording->Info();
   int y = 2;
   cTextScroller ts;
-  char t[32];
-  snprintf(t, sizeof(t), "%s  %s", *DateString(Recording->start), *TimeString(Recording->start));
+  cString t = cString::sprintf("%s  %s  %s", *DateString(Recording->Start()), *TimeString(Recording->Start()), Info->ChannelName() ? Info->ChannelName() : "");
   ts.Set(osd, 0, y, ScOsdWidth, ScOsdHeight - y - 2, t, &Font, clrYellow, clrBackground);
   y += ts.Height();
+  int xt = ScOsdWidth;
   if (Info->GetEvent()->ParentalRating()) {
      cString buffer = cString::sprintf(" %s ", *Info->GetEvent()->GetParentalRatingString());
-     osd->DrawText(ScOsdWidth - Utf8StrLen(buffer), y, buffer, clrBlack, clrYellow, &Font);
+     int w = Utf8StrLen(buffer);
+     osd->DrawText(xt - w, y, buffer, clrBlack, clrYellow, &Font);
+     xt -= w + 1;
+     }
+  if (Info->Errors() > 0) {
+     cString buffer = cString::sprintf(" %d %s ", Info->Errors(), tr("errors"));
+     int w = Utf8StrLen(buffer);
+     osd->DrawText(xt - w, y, buffer, clrBlack, clrYellow, &Font);
+     xt -= w + 1;
      }
   y += 1;
   const char *Title = Info->Title();
@@ -475,6 +492,8 @@ void cSkinCursesDisplayMenu::SetText(const char *Text, bool FixedFont)
 
 void cSkinCursesDisplayMenu::Flush(void)
 {
+  if (cVideoDiskUsage::HasChanged(lastDiskUsageState))
+     DrawTitle();
   cString date = DayDateTime();
   osd->DrawText(ScOsdWidth - Utf8StrLen(date) - 2, 0, date, clrBlack, clrCyan, &Font);
   osd->Flush();
@@ -488,15 +507,15 @@ private:
   bool message;
 public:
   cSkinCursesDisplayReplay(bool ModeOnly);
-  virtual ~cSkinCursesDisplayReplay();
-  virtual void SetTitle(const char *Title);
-  virtual void SetMode(bool Play, bool Forward, int Speed);
-  virtual void SetProgress(int Current, int Total);
-  virtual void SetCurrent(const char *Current);
-  virtual void SetTotal(const char *Total);
-  virtual void SetJump(const char *Jump);
-  virtual void SetMessage(eMessageType Type, const char *Text);
-  virtual void Flush(void);
+  virtual ~cSkinCursesDisplayReplay() override;
+  virtual void SetTitle(const char *Title) override;
+  virtual void SetMode(bool Play, bool Forward, int Speed) override;
+  virtual void SetProgress(int Current, int Total) override;
+  virtual void SetCurrent(const char *Current) override;
+  virtual void SetTotal(const char *Total) override;
+  virtual void SetJump(const char *Jump) override;
+  virtual void SetMessage(eMessageType Type, const char *Text) override;
+  virtual void Flush(void) override;
   };
 
 cSkinCursesDisplayReplay::cSkinCursesDisplayReplay(bool ModeOnly)
@@ -579,9 +598,9 @@ private:
   cOsd *osd;
 public:
   cSkinCursesDisplayVolume(void);
-  virtual ~cSkinCursesDisplayVolume();
-  virtual void SetVolume(int Current, int Total, bool Mute);
-  virtual void Flush(void);
+  virtual ~cSkinCursesDisplayVolume() override;
+  virtual void SetVolume(int Current, int Total, bool Mute) override;
+  virtual void Flush(void) override;
   };
 
 cSkinCursesDisplayVolume::cSkinCursesDisplayVolume(void)
@@ -601,6 +620,7 @@ void cSkinCursesDisplayVolume::SetVolume(int Current, int Total, bool Mute)
      osd->DrawText(0, 0, tr("Key$Mute"), clrGreen, clrBackground, &Font);
      }
   else {
+     // TRANSLATORS: note the trailing blank!
      const char *Prompt = tr("Volume ");
      int l = Utf8StrLen(Prompt);
      int p = (ScOsdWidth - l) * Current / Total;
@@ -625,10 +645,10 @@ private:
   void SetItem(const char *Text, int Index, bool Current);
 public:
   cSkinCursesDisplayTracks(const char *Title, int NumTracks, const char * const *Tracks);
-  virtual ~cSkinCursesDisplayTracks();
-  virtual void SetTrack(int Index, const char * const *Tracks);
-  virtual void SetAudioChannel(int AudioChannel) {}
-  virtual void Flush(void);
+  virtual ~cSkinCursesDisplayTracks() override;
+  virtual void SetTrack(int Index, const char * const *Tracks) override;
+  virtual void SetAudioChannel(int AudioChannel) override {}
+  virtual void Flush(void) override;
   };
 
 cSkinCursesDisplayTracks::cSkinCursesDisplayTracks(const char *Title, int NumTracks, const char * const *Tracks)
@@ -685,9 +705,9 @@ private:
   cOsd *osd;
 public:
   cSkinCursesDisplayMessage(void);
-  virtual ~cSkinCursesDisplayMessage();
-  virtual void SetMessage(eMessageType Type, const char *Text);
-  virtual void Flush(void);
+  virtual ~cSkinCursesDisplayMessage() override;
+  virtual void SetMessage(eMessageType Type, const char *Text) override;
+  virtual void Flush(void) override;
   };
 
 cSkinCursesDisplayMessage::cSkinCursesDisplayMessage(void)
@@ -715,13 +735,13 @@ void cSkinCursesDisplayMessage::Flush(void)
 class cSkinCurses : public cSkin {
 public:
   cSkinCurses(void);
-  virtual const char *Description(void);
-  virtual cSkinDisplayChannel *DisplayChannel(bool WithInfo);
-  virtual cSkinDisplayMenu *DisplayMenu(void);
-  virtual cSkinDisplayReplay *DisplayReplay(bool ModeOnly);
-  virtual cSkinDisplayVolume *DisplayVolume(void);
-  virtual cSkinDisplayTracks *DisplayTracks(const char *Title, int NumTracks, const char * const *Tracks);
-  virtual cSkinDisplayMessage *DisplayMessage(void);
+  virtual const char *Description(void) override;
+  virtual cSkinDisplayChannel *DisplayChannel(bool WithInfo) override;
+  virtual cSkinDisplayMenu *DisplayMenu(void) override;
+  virtual cSkinDisplayReplay *DisplayReplay(bool ModeOnly) override;
+  virtual cSkinDisplayVolume *DisplayVolume(void) override;
+  virtual cSkinDisplayTracks *DisplayTracks(const char *Title, int NumTracks, const char * const *Tracks) override;
+  virtual cSkinDisplayMessage *DisplayMessage(void) override;
   };
 
 cSkinCurses::cSkinCurses(void)
@@ -771,18 +791,18 @@ private:
   // Add any member variables or functions you may need here.
 public:
   cPluginSkinCurses(void);
-  virtual ~cPluginSkinCurses();
-  virtual const char *Version(void) { return VERSION; }
-  virtual const char *Description(void) { return tr(DESCRIPTION); }
-  virtual const char *CommandLineHelp(void);
-  virtual bool ProcessArgs(int argc, char *argv[]);
-  virtual bool Initialize(void);
-  virtual bool Start(void);
-  virtual void Housekeeping(void);
-  virtual const char *MainMenuEntry(void) { return tr(MAINMENUENTRY); }
-  virtual cOsdObject *MainMenuAction(void);
-  virtual cMenuSetupPage *SetupMenu(void);
-  virtual bool SetupParse(const char *Name, const char *Value);
+  virtual ~cPluginSkinCurses() override;
+  virtual const char *Version(void) override { return VERSION; }
+  virtual const char *Description(void) override { return tr(DESCRIPTION); }
+  virtual const char *CommandLineHelp(void) override;
+  virtual bool ProcessArgs(int argc, char *argv[]) override;
+  virtual bool Initialize(void) override;
+  virtual bool Start(void) override;
+  virtual void Housekeeping(void) override;
+  virtual const char *MainMenuEntry(void) override { return tr(MAINMENUENTRY); }
+  virtual cOsdObject *MainMenuAction(void) override;
+  virtual cMenuSetupPage *SetupMenu(void) override;
+  virtual bool SetupParse(const char *Name, const char *Value) override;
   };
 
 cPluginSkinCurses::cPluginSkinCurses(void)
@@ -813,12 +833,8 @@ bool cPluginSkinCurses::ProcessArgs(int argc, char *argv[])
 bool cPluginSkinCurses::Initialize(void)
 {
   // Initialize any background activities the plugin shall perform.
-  WINDOW *w = initscr();
-  if (w) {
-     ScOsdWidth  = w->_maxx - w->_begx + 1;
-     ScOsdHeight = w->_maxy - w->_begy + 1;
+  if (initscr())
      return true;
-     }
   return false;
 }
 
