@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 2.29 2010/10/24 11:22:35 kls Exp $
+ * $Id: config.h 5.34 2025/07/24 13:43:47 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,32 +22,66 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "1.7.17"
-#define VDRVERSNUM   10717  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "2.7.7"
+#define VDRVERSNUM   20707  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "1.7.17"
-#define APIVERSNUM   10717  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION      "9"
+#define APIVERSNUM   30009
 
-// When loading plugins, VDR searches them by their APIVERSION, which
-// may be smaller than VDRVERSION in case there have been no changes to
-// VDR header files since the last APIVERSION. This allows compiled
+// When loading plugins, VDR searches files by their APIVERSION, which
+// is different from VDRVERSION. APIVERSION is a plain number, incremented
+// only when there are changes to the plugin API. This allows compiled
 // plugins to work with newer versions of the core VDR as long as no
-// VDR header files have changed.
+// interfaces have changed. APIVERSNUM begins with "300.." for backwards
+// compatibility and can be used in #if preprocessor statements to handle
+// version dependent code.
 
-#define MAXPRIORITY 99
-#define MAXLIFETIME 99
+#define MAXPRIORITY       99
+#define MINPRIORITY       (-MAXPRIORITY)
+#define LIVEPRIORITY      0                  // priority used when selecting a device for live viewing
+#define TRANSFERPRIORITY  (LIVEPRIORITY - 1) // priority used for actual local Transfer Mode
+#define IDLEPRIORITY      (MINPRIORITY - 1)  // priority of an idle device
+#define MAXLIFETIME       99
+#define DEFINSTRECTIME    180 // default instant recording time (minutes)
+
+#define TIMERMACRO_TITLE    "TITLE"
+#define TIMERMACRO_EPISODE  "EPISODE"
+#define TIMERMACRO_BEFORE   "{<}"
+#define TIMERMACRO_MATCH    "{=}"
+#define TIMERMACRO_AFTER    "{>}"
+
+#define TIMERPATTERN_AVOID  "@"
+#define TIMERPATTERN_BEGIN  "^"
+#define TIMERPATTERN_END    "$"
 
 #define MINOSDWIDTH   480
 #define MAXOSDWIDTH  1920
 #define MINOSDHEIGHT  324
 #define MAXOSDHEIGHT 1200
 
-#define MaxFileName 256
+#define MaxFileName NAME_MAX // obsolete - use NAME_MAX directly instead!
 #define MaxSkinName 16
 #define MaxThemeName 16
 
+// Basically VDR works according to the DVB standard, but there are countries/providers
+// that use other standards, which in some details deviate from the DVB standard.
+// This makes it necessary to handle things differently in some areas, depending on
+// which standard is actually used. The following macros are used to distinguish
+// these cases (make sure to adjust cMenuSetupDVB::standardComplianceTexts accordingly
+// when adding a new standard):
+
+#define STANDARD_DVB       0
+#define STANDARD_ANSISCTE  1
+#define STANDARD_NORDIG    2
+
+// Automatic subtitles:
+
+#define SUBTITLES_NO     0
+#define SUBTITLES_ALWAYS 1
+#define SUBTITLES_REWIND 2
+
 typedef uint32_t in_addr_t; //XXX from /usr/include/netinet/in.h (apparently this is not defined on systems with glibc < 2.2)
 
 class cSVDRPhost : public cListObject {
@@ -61,6 +95,25 @@ public:
   bool Accepts(in_addr_t Address);
   };
 
+class cSatCableNumbers {
+private:
+  int size;
+  int *array;
+public:
+  cSatCableNumbers(int Size, const char *s = NULL);
+  ~cSatCableNumbers();
+  int Size(void) const { return size; }
+  int *Array(void) { return array; }
+  bool FromString(const char *s);
+  cString ToString(void);
+  int FirstDeviceIndex(int DeviceIndex) const;
+      ///< Returns the first device index (starting at 0) that uses the same
+      ///< sat cable number as the device with the given DeviceIndex.
+      ///< If the given device does not use the same sat cable as any other device,
+      ///< or if the resulting value would be the same as DeviceIndex,
+      ///< or if DeviceIndex is out of range, -1 is returned.
+  };
+
 template<class T> class cConfig : public cList<T> {
 private:
   char *fileName;
@@ -72,8 +125,8 @@ private:
     cList<T>::Clear();
   }
 public:
-  cConfig(void) { fileName = NULL; }
-  virtual ~cConfig() { free(fileName); }
+  cConfig(const char *NeedsLocking = NULL): cList<T>(NeedsLocking) { fileName = NULL; }
+  virtual ~cConfig() override { free(fileName); }
   const char *FileName(void) { return fileName; }
   bool Load(const char *FileName = NULL, bool AllowComments = false, bool MustExist = false)
   {
@@ -103,7 +156,7 @@ public:
                 if (!isempty(s)) {
                    T *l = new T;
                    if (l->Parse(s))
-                      Add(l);
+                      this->Add(l);
                    else {
                       esyslog("ERROR: error in %s, line %d", fileName, line);
                       delete l;
@@ -122,7 +175,7 @@ public:
        fprintf(stderr, "vdr: error while reading '%s'\n", fileName);
     return result;
   }
-  bool Save(void)
+  bool Save(void) const
   {
     bool result = true;
     T *l = (T *)this->First();
@@ -150,8 +203,8 @@ private:
   cList<cNestedItem> *subItems;
 public:
   cNestedItem(const char *Text, bool WithSubItems = false);
-  virtual ~cNestedItem();
-  virtual int Compare(const cListObject &ListObject) const;
+  virtual ~cNestedItem() override;
+  virtual int Compare(const cListObject &ListObject) const override;
   const char *Text(void) const { return text; }
   cList<cNestedItem> *SubItems(void) { return subItems; }
   void AddSubItem(cNestedItem *Item);
@@ -166,7 +219,7 @@ private:
   bool Write(FILE *f, cList<cNestedItem> *List, int Indent = 0);
 public:
   cNestedItemList(void);
-  virtual ~cNestedItemList();
+  virtual ~cNestedItemList() override;
   void Clear(void);
   bool Load(const char *FileName);
   bool Save(void);
@@ -191,8 +244,8 @@ private:
 public:
   cSetupLine(void);
   cSetupLine(const char *Name, const char *Value, const char *Plugin = NULL);
-  virtual ~cSetupLine();
-  virtual int Compare(const cListObject &ListObject) const;
+  virtual ~cSetupLine() override;
+  virtual int Compare(const cListObject &ListObject) const override;
   const char *Plugin(void) { return plugin; }
   const char *Name(void) { return name; }
   const char *Value(void) { return value; }
@@ -223,15 +276,22 @@ public:
   int MenuScrollWrap;
   int MenuKeyCloses;
   int MarkInstantRecord;
-  char NameInstantRecord[MaxFileName];
+  char NameInstantRecord[NAME_MAX + 1];
   int InstantRecordTime;
   int LnbSLOF;
   int LnbFrequLo;
   int LnbFrequHi;
   int DiSEqC;
+  int UsePositioner;
+  int SiteLat;
+  int SiteLon;
+  int PositionerSpeed;
+  int PositionerSwing;
+  int PositionerLastLon;
   int SetSystemTime;
   int TimeSource;
   int TimeTransponder;
+  int StandardCompliance;
   int MarginStart, MarginStop;
   int AudioLanguages[I18N_MAX_LANGUAGES + 1];
   int DisplaySubtitles;
@@ -239,22 +299,33 @@ public:
   int SubtitleOffset;
   int SubtitleFgTransparency, SubtitleBgTransparency;
   int EPGLanguages[I18N_MAX_LANGUAGES + 1];
+  int EPGScanMaxChannel;
+  int EPGPauseAfterScan;
   int EPGScanTimeout;
   int EPGBugfixLevel;
   int EPGLinger;
   int SVDRPTimeout;
+  int SVDRPPeering;
+  char SVDRPHostName[HOST_NAME_MAX];
+  char SVDRPDefaultHost[HOST_NAME_MAX];
   int ZapTimeout;
   int ChannelEntryTimeout;
-  int PrimaryLimit;
+  int RcRepeatDelay;
+  int RcRepeatDelta;
   int DefaultPriority, DefaultLifetime;
-  int PausePriority, PauseLifetime;
+  int RecordKeyHandling;
   int PauseKeyHandling;
+  int PausePriority, PauseLifetime;
   int UseSubtitle;
   int UseVps;
   int VpsMargin;
   int RecordingDirs;
   int FoldersInTimerMenu;
+  int AlwaysSortFoldersFirst;
+  int DefaultSortModeRec;
+  int RecSortingDirection;
   int NumberKeysForChars;
+  int ColorKey0, ColorKey1, ColorKey2, ColorKey3;
   int VideoDisplayFormat;
   int VideoFormat;
   int UpdateChannels;
@@ -283,15 +354,31 @@ public:
   time_t NextWakeupTime;
   int MultiSpeedMode;
   int ShowReplayMode;
+  int ShowRemainingTime;
+  int ProgressDisplayTime;
+  int PauseOnMarkSet;
+  int PauseOnMarkJump;
+  int SkipEdited;
+  int PauseAtLastMark;
+  int AdaptiveSkipInitial;
+  int AdaptiveSkipTimeout;
+  int AdaptiveSkipAlternate;
+  int AdaptiveSkipPrevNext;
+  int SkipSeconds;
+  int SkipSecondsRepeat;
   int ResumeID;
   int CurrentChannel;
   int CurrentVolume;
+  int VolumeSteps;
+  int VolumeLinearize;
   int CurrentDolby;
-  int InitialChannel;
   int InitialVolume;
   int ChannelsWrap;
+  int ShowChannelNamesWithSource;
   int EmergencyExit;
   int __EndData__;
+  cString InitialChannel;
+  cString DeviceBondings;
   cSetup(void);
   cSetup& operator= (const cSetup &s);
   bool Load(const char *FileName);
