@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbci.h 1.1 2006/11/26 11:19:42 kls Exp $
+ * $Id: dvbci.h 5.1 2025/03/02 11:03:35 kls Exp $
  */
 
 #ifndef __DVBCI_H
@@ -17,14 +17,14 @@ private:
   cDevice *device;
   int fd;
 protected:
-  virtual int Read(uint8_t *Buffer, int MaxLength);
-  virtual void Write(const uint8_t *Buffer, int Length);
-  virtual bool Reset(int Slot);
-  virtual eModuleStatus ModuleStatus(int Slot);
-  virtual bool Assign(cDevice *Device, bool Query = false);
+  virtual int Read(uint8_t *Buffer, int MaxLength) override;
+  virtual void Write(const uint8_t *Buffer, int Length) override;
+  virtual bool Reset(int Slot) override;
+  virtual eModuleStatus ModuleStatus(int Slot) override;
+  virtual bool Assign(cDevice *Device, bool Query = false) override;
   cDvbCiAdapter(cDevice *Device, int Fd);
 public:
-  virtual ~cDvbCiAdapter();
+  virtual ~cDvbCiAdapter() override;
   static cDvbCiAdapter *CreateCiAdapter(cDevice *Device, int Fd);
   };
 
