@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.h 2.11 2011/03/12 16:06:48 kls Exp $
+ * $Id: osd.h 5.3 2025/03/02 11:03:35 kls Exp $
  */
 
 #ifndef __OSD_H
@@ -25,6 +25,7 @@
 #define ALPHA_TRANSPARENT  0x00
 #define ALPHA_OPAQUE       0xFF
 #define IS_OPAQUE(c)       ((c >> 24) == ALPHA_OPAQUE)
+#define TEXT_ALIGN_BORDER  10 // fraction of the font height used for sizing border
 
 enum {
                    //AARRGGBB
@@ -69,6 +70,14 @@ inline tColor RgbToColor(double R, double G, double B)
   return RgbToColor(uint8_t(0xFF * R), uint8_t(0xFF * G), uint8_t(0xFF * B));
 }
 
+tColor RgbShade(tColor Color, double Factor);
+   ///< Returns a brighter (Factor > 0) or darker (Factor < 0) version
+   ///< of the given Color.
+   ///< If Factor is 0.0, the return value is the unchanged Color,
+   ///< If Factor is 1.0, white is returned.
+   ///< If Factor is -1.0, black is returned.
+   ///< The alpha value of Color is returned unchanged.
+
 tColor HsvToColor(double H, double S, double V);
    ///< Converts the given Hue (0..360), Saturation (0..1) and Value (0..1)
    ///< to an RGB tColor value. The alpha value of the result is 0x00, so
@@ -151,6 +160,7 @@ enum eTextAlignment { taCenter  = 0x00,
                       taRight   = 0x02,
                       taTop     = 0x04,
                       taBottom  = 0x08,
+                      taBorder  = 0x10, // keeps some distance from the left or right alignment edge
                       taDefault = taTop | taLeft
                     };
 
@@ -172,7 +182,7 @@ public:
        ///< Creates a bitmap and loads an XPM image from the given file.
   cBitmap(const char *const Xpm[]);
        ///< Creates a bitmap from the given XPM data.
-  virtual ~cBitmap();
+  virtual ~cBitmap() override;
   int X0(void) const { return x0; }
   int Y0(void) const { return y0; }
   int Width(void) const { return width; }
@@ -182,6 +192,8 @@ public:
        ///< contents of the bitmap will be lost. If Width and Height are the same
        ///< as the current values, nothing will happen and the bitmap remains
        ///< unchanged.
+  void SetOffset(int X0, int Y0) { x0 = X0; y0 = Y0; }
+       ///< Sets the offset of this bitmap to the given values.
   bool Contains(int x, int y) const;
        ///< Returns true if this bitmap contains the point (x, y).
   bool Covers(int x1, int y1, int x2, int y2) const;
@@ -211,6 +223,8 @@ public:
   void SetIndex(int x, int y, tIndex Index);
        ///< Sets the index at the given coordinates to Index.
        ///< Coordinates are relative to the bitmap's origin.
+  void Fill(tIndex Index);
+       ///< Fills the bitmap data with the given Index.
   void DrawPixel(int x, int y, tColor Color);
        ///< Sets the pixel at the given coordinates to the given Color, which is
        ///< a full 32 bit ARGB value.
@@ -243,7 +257,7 @@ public:
        ///< 0       draws the entire ellipse
        ///< 1..4    draws only the first, second, third or fourth quadrant, respectively
        ///< 5..8    draws the right, top, left or bottom half, respectively
-       ///< -1..-8  draws the inverted part of the given quadrant(s)
+       ///< -1..-4  draws the inverted part of the given quadrant
        ///< If Quadrants is not 0, the coordinates are those of the actual area, not
        ///< the full circle!
   void DrawSlope(int x1, int y1, int x2, int y2, tColor Color, int Type);
@@ -273,8 +287,11 @@ public:
        ///< the 2^NewBpp most frequently used colors as defined in the current palette.
        ///< If NewBpp is not smaller than the bitmap's current color depth,
        ///< or if it is not one of 4bpp or 2bpp, nothing happens.
-  cBitmap *Scale(double FactorX, double FactorY);
+  cBitmap *Scaled(double FactorX, double FactorY, bool AntiAlias = false) const;
        ///< Creates a copy of this bitmap, scaled by the given factors.
+       ///< If AntiAlias is true and either of the factors is greater than 1.0,
+       ///< anti-aliasing is applied. This will also set the color depth of the
+       ///< returned bitmap to 8bpp.
        ///< The caller must delete the returned bitmap once it is no longer used.
   };
 
@@ -430,6 +447,11 @@ public:
        ///< Clears the image data by setting all pixels to be fully transparent.
   void Fill(tColor Color);
        ///< Fills the image data with the given Color.
+  cImage *Scaled(double FactorX, double FactorY, bool AntiAlias = false) const;
+       ///< Creates a copy of this image, scaled by the given factors.
+       ///< If AntiAlias is true and either of the factors is greater than 1.0,
+       ///< anti-aliasing is applied.
+       ///< The caller must delete the returned image once it is no longer used.
   };
 
 #define MAXPIXMAPLAYERS    8
@@ -493,8 +515,10 @@ public:
        ///< In order to allow devices that can handle only a limited number of layers,
        ///< the Layer parameter must be less than 8 (MAXPIXMAPLAYERS).
        ///< ViewPort defines the rectangle in which this pixmap will be rendered on
-       ///< the OSD. If no DrawPort ist given, it defaults to the same size as the
-       ///< ViewPort, with its upper left corner set to (0, 0).
+       ///< the OSD. The coordinate (0, 0) corresponds to the upper left corner of the
+       ///< OSD. If no DrawPort is given, it defaults to the same size as the
+       ///< ViewPort, with its upper left corner set to (0, 0). The DrawPort's origin
+       ///< is relative to the ViewPort's origin.
        ///< All drawing operations will be executed relative to the origin of the
        ///< DrawPort rectangle, and will be clipped to the size of this rectangle.
        ///< The DrawPort may have a different size than the ViewPort. If it is smaller
@@ -505,6 +529,8 @@ public:
        ///< intersects with the ViewPort will be visible on the OSD.
        ///< The drawing area of a newly created cPixmap is not initialized and may
        ///< contain random data.
+       ///< See cOsd::MaxPixmapSize() for information on the maximum size of pixmaps
+       ///< supported by the system.
   static void Lock(void) { mutex.Lock(); }
        ///< All public member functions of cPixmap set locks as necessary to make sure
        ///< they are thread-safe (unless noted otherwise). If several cPixmap member
@@ -563,7 +589,7 @@ public:
        ///< Sets the pixmap's draw port to the given Point.
        ///< Only the origin point of the draw port can be modified, its size is fixed.
        ///< By default, setting a new draw port point results in marking the relevant
-       ///< part of the view port as "drity". If Dirty is set to false, the view port
+       ///< part of the view port as "dirty". If Dirty is set to false, the view port
        ///< will not be marked as dirty. This is mainly used to implement the Pan()
        ///< function.
        ///< If a derived class reimplements this function, it needs to call the base
@@ -581,11 +607,32 @@ public:
        ///< the given Point. ImageHandle must be a value that has previously been
        ///< returned by a call to cOsdProvider::StoreImage(). If ImageHandle
        ///< has an invalid value, nothing happens.
+  virtual void DrawScaledImage(const cPoint &Point, const cImage &Image, double FactorX, double FactorY, bool AntiAlias = false) {};
+       ///< Draws the given Image into this pixmap at the given Point and scales it.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< This function should be 'pure', just like all the others. However, since it was
+       ///< introduced when this class was already in widespread use, the default was made
+       ///< empty, so that existing code will still compile.
+  virtual void DrawScaledImage(const cPoint &Point, int ImageHandle, double FactorX, double FactorY, bool AntiAlias = false) {};
+       ///< Draws the image referenced by the given ImageHandle into this pixmap at
+       ///< the given Point and scales it. ImageHandle must be a value that has
+       ///< previously been returned by a call to cOsdProvider::StoreImage().
+       ///< If ImageHandle has an invalid value, nothing happens.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< This function should be 'pure', just like all the others. However, since it was
+       ///< introduced when this class was already in widespread use, the default was made
+       ///< empty, so that existing code will still compile.
   virtual void DrawPixel(const cPoint &Point, tColor Color) = 0;
        ///< Sets the pixel at the given Point to the given Color, which is
        ///< a full 32 bit ARGB value. If the alpha value of Color is not 0xFF
        ///< (fully opaque), and this is the background pixmap (layer 0), the pixel is
        ///< alpha blended with the existing color at the given position in this pixmap.
+  virtual void DrawBlendedPixel(const cPoint &Point, tColor Color, uint8_t AlphaLayer = ALPHA_OPAQUE) { DrawPixel(Point, Color); }
+       ///< Like DrawPixel(), but with an additional AlphaLayer, and works on any
+       ///< pixmap, not only the background. The default implementation just calls
+       ///< DrawPixel(), ignoring AlphaLayer.
   virtual void DrawBitmap(const cPoint &Point, const cBitmap &Bitmap, tColor ColorFg = 0, tColor ColorBg = 0, bool Overlay = false) = 0;
        ///< Sets the pixels in the OSD with the data from the given
        ///< Bitmap, putting the upper left corner of the Bitmap at Point.
@@ -611,7 +658,7 @@ public:
        ///< 0       draws the entire ellipse
        ///< 1..4    draws only the first, second, third or fourth quadrant, respectively
        ///< 5..8    draws the right, top, left or bottom half, respectively
-       ///< -1..-8  draws the inverted part of the given quadrant(s)
+       ///< -1..-4  draws the inverted part of the given quadrant
        ///< If Quadrants is not 0, the coordinates are those of the actual area, not
        ///< the full circle!
   virtual void DrawSlope(const cRect &Rect, tColor Color, int Type) = 0;
@@ -644,7 +691,7 @@ public:
        ///< covers the entire view port. This may be of advantage if, e.g.,
        ///< there is a draw port that holds, say, 11 lines of text, while the
        ///< view port displays only 10 lines. By Pan()'ing the draw port up one
-       ///< line, an new bottom line can be written into the draw port (without
+       ///< line, a new bottom line can be written into the draw port (without
        ///< being seen through the view port), and later the draw port can be
        ///< shifted smoothly, resulting in a smooth scrolling.
        ///< It is the caller's responsibility to make sure that Source and Dest
@@ -669,26 +716,28 @@ private:
 public:
   cPixmapMemory(void);
   cPixmapMemory(int Layer, const cRect &ViewPort, const cRect &DrawPort = cRect::Null);
-  virtual ~cPixmapMemory();
+  virtual ~cPixmapMemory() override;
   const uint8_t *Data(void) { return (uint8_t *)data; }
-  virtual void Clear(void);
-  virtual void Fill(tColor Color);
-  virtual void DrawImage(const cPoint &Point, const cImage &Image);
-  virtual void DrawImage(const cPoint &Point, int ImageHandle);
-  virtual void DrawPixel(const cPoint &Point, tColor Color);
-  virtual void DrawBitmap(const cPoint &Point, const cBitmap &Bitmap, tColor ColorFg = 0, tColor ColorBg = 0, bool Overlay = false);
-  virtual void DrawText(const cPoint &Point, const char *s, tColor ColorFg, tColor ColorBg, const cFont *Font, int Width = 0, int Height = 0, int Alignment = taDefault);
-  virtual void DrawRectangle(const cRect &Rect, tColor Color);
-  virtual void DrawEllipse(const cRect &Rect, tColor Color, int Quadrants = 0);
-  virtual void DrawSlope(const cRect &Rect, tColor Color, int Type);
-  virtual void Render(const cPixmap *Pixmap, const cRect &Source, const cPoint &Dest);
-  virtual void Copy(const cPixmap *Pixmap, const cRect &Source, const cPoint &Dest);
-  virtual void Scroll(const cPoint &Dest, const cRect &Source = cRect::Null);
-  virtual void Pan(const cPoint &Dest, const cRect &Source = cRect::Null);
+  virtual void Clear(void) override;
+  virtual void Fill(tColor Color) override;
+  virtual void DrawImage(const cPoint &Point, const cImage &Image) override;
+  virtual void DrawImage(const cPoint &Point, int ImageHandle) override;
+  virtual void DrawScaledImage(const cPoint &Point, const cImage &Image, double FactorX, double FactorY, bool AntiAlias = false) override;
+  virtual void DrawScaledImage(const cPoint &Point, int ImageHandle, double FactorX, double FactorY, bool AntiAlias = false) override;
+  virtual void DrawPixel(const cPoint &Point, tColor Color) override;
+  virtual void DrawBlendedPixel(const cPoint &Point, tColor Color, uint8_t AlphaLayer = ALPHA_OPAQUE) override;
+  virtual void DrawBitmap(const cPoint &Point, const cBitmap &Bitmap, tColor ColorFg = 0, tColor ColorBg = 0, bool Overlay = false) override;
+  virtual void DrawText(const cPoint &Point, const char *s, tColor ColorFg, tColor ColorBg, const cFont *Font, int Width = 0, int Height = 0, int Alignment = taDefault) override;
+  virtual void DrawRectangle(const cRect &Rect, tColor Color) override;
+  virtual void DrawEllipse(const cRect &Rect, tColor Color, int Quadrants = 0) override;
+  virtual void DrawSlope(const cRect &Rect, tColor Color, int Type) override;
+  virtual void Render(const cPixmap *Pixmap, const cRect &Source, const cPoint &Dest) override;
+  virtual void Copy(const cPixmap *Pixmap, const cRect &Source, const cPoint &Dest) override;
+  virtual void Scroll(const cPoint &Dest, const cRect &Source = cRect::Null) override;
+  virtual void Pan(const cPoint &Dest, const cRect &Source = cRect::Null) override;
   };
 
 #define MAXOSDAREAS 16
-#define MAXOSDPIXMAPS 64
 
 /// The cOsd class is the interface to the "On Screen Display".
 /// An actual output device needs to derive from this class and implement
@@ -698,7 +747,7 @@ public:
 /// reimplement all necessary cPixmap functions and do the rendering
 /// itself ("high level mode").
 /// If an OSD provides a "high level mode", it shall also provide a "raw mode"
-/// in order to verify proper operation. The plugin that impements the OSD
+/// in order to verify proper operation. The plugin that implements the OSD
 /// shall offer a configuration switch in its setup.
 
 class cOsd {
@@ -706,13 +755,14 @@ class cOsd {
 private:
   static int osdLeft, osdTop, osdWidth, osdHeight;
   static cVector<cOsd *> Osds;
+  static cSize maxPixmapSize;
+  static cMutex mutex;
   bool isTrueColor;
   cBitmap *savedBitmap;
   cBitmap *bitmaps[MAXOSDAREAS];
   int numBitmaps;
   cPixmapMemory *savedPixmap;
-  cPixmap *pixmaps[MAXOSDPIXMAPS];
-  int numPixmaps;
+  cVector<cPixmap *> pixmaps;
   int left, top, width, height;
   uint level;
   bool active;
@@ -721,7 +771,7 @@ protected:
        ///< Initializes the OSD with the given coordinates.
        ///< By default it is assumed that the full area will be able to display
        ///< full 32 bit graphics (ARGB with eight bit for each color and the alpha
-       ///< value, repectively). However, the actual hardware in use may not be
+       ///< value, respectively). However, the actual hardware in use may not be
        ///< able to display such a high resolution OSD, so there is an option to
        ///< divide the full OSD area into several sub-areas with lower color depths
        ///< and individual palettes. The sub-areas need not necessarily cover the
@@ -741,22 +791,19 @@ protected:
   virtual void SetActive(bool On) { active = On; }
        ///< Sets this OSD to be the active one.
        ///< A derived class must call cOsd::SetActive(On).
-  const cPixmap * const *Pixmaps(void) { return pixmaps; }
-       ///< Returns the list of currently active pixmaps in this OSD.
-  int NumPixmaps(void) { return numPixmaps; }
-       ///< Returns the number of currently active pixmaps in this OSD.
   cPixmap *AddPixmap(cPixmap *Pixmap);
        ///< Adds the given Pixmap to the list of currently active pixmaps in this OSD.
-       ///< Returns Pixmap if the operation was successful, or NULL if the maximum
-       ///< number of pixmaps has been exceeded.
+       ///< Returns Pixmap if the operation was successful, or NULL if for some reason
+       ///< the pixmap could not be added to the list.
        ///< A derived class that implements its own cPixmap class must call AddPixmap()
        ///< in order to add a newly created pixmap to the OSD's list of pixmaps.
-  cPixmapMemory *RenderPixmaps(void);
+  cPixmap *RenderPixmaps(void);
        ///< Renders the dirty part of all pixmaps into a resulting pixmap that
        ///< shall be displayed on the OSD. The returned pixmap's view port is
        ///< set to the location of the rectangle on the OSD that needs to be
        ///< refreshed; its draw port's origin is at (0, 0), and it has the same
        ///< size as the view port.
+       ///< Only pixmaps with a non-negative layer value are rendered.
        ///< If there are several non-overlapping dirty rectangles from different pixmaps,
        ///< they are returned separately in order to avoid re-rendering large parts
        ///< of the OSD that haven't changed at all. The caller must therefore call
@@ -766,7 +813,15 @@ protected:
        ///< by putting a LOCK_PIXMAPS into the scope of the operation).
        ///< If there are no dirty pixmaps, or if this is not a true color OSD,
        ///< this function returns NULL.
-       ///< The caller must delete the returned pixmap after use.
+       ///< The caller must call DestroyPixmap() for the returned pixmap after use.
+  cBitmap *GetBitmap(int Area);
+       ///< Returns a pointer to the bitmap for the given Area, or NULL if no
+       ///< such bitmap exists.
+       ///< If this is a true color OSD, a pointer to a dummy bitmap with 8bpp
+       ///< is returned. This is done so that skins that call this function
+       ///< in order to preset the bitmap's palette won't crash.
+       ///< Use of this function outside of derived classes is deprecated and it
+       ///< may be made 'protected' in a future version.
 public:
   virtual ~cOsd();
        ///< Shuts down the OSD.
@@ -799,18 +854,19 @@ public:
        ///< requested colors. By default the palette assumes there will be
        ///< 10 fixed colors and 10 color combinations.
        ///< If this is a true color OSD, this function does nothing.
-  cBitmap *GetBitmap(int Area);
-       ///< Returns a pointer to the bitmap for the given Area, or NULL if no
-       ///< such bitmap exists.
-       ///< If this is a true color OSD, a pointer to a dummy bitmap with 8bpp
-       ///< is returned. This is done so that skins that call this function
-       ///< in order to preset the bitmap's palette won't crash.
+  virtual const cSize &MaxPixmapSize(void) const;
+       ///< Returns the maximum possible size of a pixmap this OSD can create.
+       ///< Derived classes can reimplement this function if their implementation
+       ///< of cPixmap can only provide pixmaps up to a certain size.
+       ///< The default implementation returns a cSize object of maximal size
+       ///< (INT_MAX). However, memory restrictions may still apply.
   virtual cPixmap *CreatePixmap(int Layer, const cRect &ViewPort, const cRect &DrawPort = cRect::Null);
        ///< Creates a new true color pixmap on this OSD (see cPixmap for details).
        ///< The caller must not delete the returned object, it will be deleted when
        ///< the OSD is deleted. DestroyPixmap() can be called if a pixmap shall be
        ///< destroyed before the OSD is deleted.
-       ///< If this is not a true color OSD, this function returns NULL.
+       ///< If this is not a true color OSD, or if the pixmap could not be created
+       ///< due to limited resources, this function returns NULL.
   virtual void DestroyPixmap(cPixmap *Pixmap);
        ///< Destroys the given Pixmap, which has previously been created by a call to
        ///< CreatePixmap(). When the OSD is deleted, all pixmaps are destroyed
@@ -825,6 +881,19 @@ public:
        ///< returned by a call to cOsdProvider::StoreImage(). If ImageHandle
        ///< has an invalid value, nothing happens.
        ///< If this is not a true color OSD, this function does nothing.
+  virtual void DrawScaledImage(const cPoint &Point, const cImage &Image, double FactorX, double FactorY, bool AntiAlias = false);
+       ///< Draws the given Image on this OSD at the given Point and scales it.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< If this is not a true color OSD, this function does nothing.
+  virtual void DrawScaledImage(const cPoint &Point, int ImageHandle, double FactorX, double FactorY, bool AntiAlias = false);
+       ///< Draws the image referenced by the given ImageHandle on this OSD at
+       ///< the given Point and scales it. ImageHandle must be a value that has
+       ///< previously been returned by a call to cOsdProvider::StoreImage().
+       ///< If ImageHandle has an invalid value, nothing happens.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< If this is not a true color OSD, this function does nothing.
   virtual eOsdError CanHandleAreas(const tArea *Areas, int NumAreas);
        ///< Checks whether the OSD can display the given set of sub-areas.
        ///< The return value indicates whether a call to SetAreas() with this
@@ -872,6 +941,11 @@ public:
        ///< If Overlay is true, any pixel in Bitmap that has color index 0 will
        ///< not overwrite the corresponding pixel in the target area.
        ///< If this is a true color OSD, ReplacePalette has no meaning.
+  virtual void DrawScaledBitmap(int x, int y, const cBitmap &Bitmap, double FactorX, double FactorY, bool AntiAlias = false);
+       ///< Sets the pixels in the OSD with the data from the given Bitmap, putting
+       ///< the upper left corner of the Bitmap at (x, y) and scaled by the given
+       ///< factors. If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
   virtual void DrawText(int x, int y, const char *s, tColor ColorFg, tColor ColorBg, const cFont *Font, int Width = 0, int Height = 0, int Alignment = taDefault);
        ///< Draws the given string at coordinates (x, y) with the given foreground
        ///< and background color and font. If Width and Height are given, the text
@@ -888,7 +962,7 @@ public:
        ///< 0       draws the entire ellipse
        ///< 1..4    draws only the first, second, third or fourth quadrant, respectively
        ///< 5..8    draws the right, top, left or bottom half, respectively
-       ///< -1..-8  draws the inverted part of the given quadrant(s)
+       ///< -1..-4  draws the inverted part of the given quadrant
        ///< If Quadrants is not 0, the coordinates are those of the actual area, not
        ///< the full circle!
   virtual void DrawSlope(int x1, int y1, int x2, int y2, tColor Color, int Type);
@@ -910,13 +984,16 @@ public:
        ///< pixmaps, the Flush() function should basically do something like this:
        ///<
        ///<  LOCK_PIXMAPS;
-       ///<  while (cPixmapMemory *pm = RenderPixmaps()) {
+       ///<  while (cPixmapMemory *pm = dynamic_cast<cPixmapMemory *>(RenderPixmaps())) {
        ///<        int w = pm->ViewPort().Width();
        ///<        int h = pm->ViewPort().Height();
        ///<        int d = w * sizeof(tColor);
        ///<        MyOsdDrawPixmap(Left() + pm->ViewPort().X(), Top() + pm->ViewPort().Y(), pm->Data(), w, h, h * d);
-       ///<        delete pm;
+       ///<        DestroyPixmap(pm);
        ///<        }
+       ///<
+       ///< If a plugin uses a derived cPixmap implementation, it needs to use that
+       ///< type instead of cPixmapMemory.
   };
 
 #define MAXOSDIMAGES 64
@@ -929,6 +1006,7 @@ private:
   static int oldHeight;
   static double oldAspect;
   static cImage *images[MAXOSDIMAGES];
+  static int osdState;
 protected:
   virtual cOsd *CreateOsd(int Left, int Top, uint Level) = 0;
       ///< Returns a pointer to a newly created cOsd object, which will be located
@@ -965,6 +1043,12 @@ public:
       ///< font sizes accordingly. If Force is true, all settings are recalculated,
       ///< even if the video resolution hasn't changed since the last call to
       ///< this function.
+  static bool OsdSizeChanged(int &State);
+      ///< Checks if the OSD size has changed and a currently displayed OSD needs to
+      ///< be redrawn. An internal reference value is incremented on every size change
+      ///< and is compared against State when calling the method.
+      ///< OsdSizeChanged() can be called with an uninitialized State to just get
+      ///< the current value of State.
   static bool SupportsTrueColor(void);
       ///< Returns true if the current OSD provider is able to handle a true color OSD.
   static int StoreImage(const cImage &Image);
