@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: pat.c 2.15 2010/11/07 13:47:16 kls Exp $
+ * $Id: pat.c 5.5 2025/03/02 11:03:35 kls Exp $
  */
 
 #include "pat.h"
@@ -12,23 +12,24 @@
 #include "channels.h"
 #include "libsi/section.h"
 #include "libsi/descriptor.h"
-#include "thread.h"
 
-#define PMT_SCAN_TIMEOUT  10 // seconds
+#define PMT_SCAN_TIMEOUT  1000 // ms
 
 // --- cCaDescriptor ---------------------------------------------------------
 
 class cCaDescriptor : public cListObject {
 private:
   int caSystem;
+  int caPid;
   int esPid;
   int length;
   uchar *data;
 public:
   cCaDescriptor(int CaSystem, int CaPid, int EsPid, int Length, const uchar *Data);
-  virtual ~cCaDescriptor();
+  virtual ~cCaDescriptor() override;
   bool operator== (const cCaDescriptor &arg) const;
   int CaSystem(void) { return caSystem; }
+  int CaPid(void) { return caPid; }
   int EsPid(void) { return esPid; }
   int Length(void) const { return length; }
   const uchar *Data(void) const { return data; }
@@ -37,6 +38,7 @@ public:
 cCaDescriptor::cCaDescriptor(int CaSystem, int CaPid, int EsPid, int Length, const uchar *Data)
 {
   caSystem = CaSystem;
+  caPid = CaPid;
   esPid = EsPid;
   length = Length + 6;
   data = MALLOC(uchar, length);
@@ -67,34 +69,38 @@ private:
   int source;
   int transponder;
   int serviceId;
+  int pmtPid; // needed for OctopusNet - otherwise irrelevant!
   int numCaIds;
   int caIds[MAXCAIDS + 1];
   cList<cCaDescriptor> caDescriptors;
   void AddCaId(int CaId);
 public:
-  cCaDescriptors(int Source, int Transponder, int ServiceId);
+  cCaDescriptors(int Source, int Transponder, int ServiceId, int PmtPid);
   bool operator== (const cCaDescriptors &arg) const;
   bool Is(int Source, int Transponder, int ServiceId);
   bool Is(cCaDescriptors * CaDescriptors);
   bool Empty(void) { return caDescriptors.Count() == 0; }
   void AddCaDescriptor(SI::CaDescriptor *d, int EsPid);
-  int GetCaDescriptors(const int *CaSystemIds, int BufSize, uchar *Data, int EsPid);
+  void GetCaDescriptors(const int *CaSystemIds, cDynamicBuffer &Buffer, int EsPid);
+  int GetCaPids(const int *CaSystemIds, int BufSize, int *Pids);
+  const int GetPmtPid(void) { return pmtPid; };
   const int *CaIds(void) { return caIds; }
   };
 
-cCaDescriptors::cCaDescriptors(int Source, int Transponder, int ServiceId)
+cCaDescriptors::cCaDescriptors(int Source, int Transponder, int ServiceId, int PmtPid)
 {
   source = Source;
   transponder = Transponder;
   serviceId = ServiceId;
+  pmtPid = PmtPid;
   numCaIds = 0;
   caIds[0] = 0;
 }
 
 bool cCaDescriptors::operator== (const cCaDescriptors &arg) const
 {
-  cCaDescriptor *ca1 = caDescriptors.First();
-  cCaDescriptor *ca2 = arg.caDescriptors.First();
+  const cCaDescriptor *ca1 = caDescriptors.First();
+  const cCaDescriptor *ca2 = arg.caDescriptors.First();
   while (ca1 && ca2) {
         if (!(*ca1 == *ca2))
            return false;
@@ -153,28 +159,42 @@ void cCaDescriptors::AddCaDescriptor(SI::CaDescriptor *d, int EsPid)
 // =0 - common CaDescriptor
 // <0 - all CaDescriptors regardless of type (old default)
 
-int cCaDescriptors::GetCaDescriptors(const int *CaSystemIds, int BufSize, uchar *Data, int EsPid)
+void cCaDescriptors::GetCaDescriptors(const int *CaSystemIds, cDynamicBuffer &Buffer, int EsPid)
+{
+  Buffer.Clear();
+  if (!CaSystemIds || !*CaSystemIds)
+     return;
+  for (cCaDescriptor *d = caDescriptors.First(); d; d = caDescriptors.Next(d)) {
+      if (EsPid < 0 || d->EsPid() == EsPid) {
+         const int *caids = CaSystemIds;
+         do {
+            if (*caids == 0xFFFF || d->CaSystem() == *caids)
+               Buffer.Append(d->Data(), d->Length());
+            } while (*++caids);
+         }
+      }
+}
+
+int cCaDescriptors::GetCaPids(const int *CaSystemIds, int BufSize, int *Pids)
 {
   if (!CaSystemIds || !*CaSystemIds)
      return 0;
-  if (BufSize > 0 && Data) {
-     int length = 0;
+  if (BufSize > 0 && Pids) {
+     int numPids = 0;
      for (cCaDescriptor *d = caDescriptors.First(); d; d = caDescriptors.Next(d)) {
-         if (EsPid < 0 || d->EsPid() == EsPid) {
-            const int *caids = CaSystemIds;
-            do {
-               if (d->CaSystem() == *caids) {
-                  if (length + d->Length() <= BufSize) {
-                     memcpy(Data + length, d->Data(), d->Length());
-                     length += d->Length();
-                     }
-                  else
-                     return -1;
+         const int *caids = CaSystemIds;
+         do {
+            if (*caids == 0xFFFF || d->CaSystem() == *caids) {
+               if (numPids + 1 < BufSize) {
+                  Pids[numPids++] = d->CaPid();
+                  Pids[numPids] = 0;
                   }
-               } while (*++caids);
-            }
+               else
+                  return -1;
+               }
+            } while (*++caids);
          }
-     return length;
+     return numPids;
      }
   return -1;
 }
@@ -189,7 +209,9 @@ public:
       // Returns 0 if this is an already known descriptor,
       // 1 if it is an all new descriptor with actual contents,
       // and 2 if an existing descriptor was changed.
-  int GetCaDescriptors(int Source, int Transponder, int ServiceId, const int *CaSystemIds, int BufSize, uchar *Data, int EsPid);
+  void GetCaDescriptors(int Source, int Transponder, int ServiceId, const int *CaSystemIds, cDynamicBuffer &Buffer, int EsPid);
+  int GetCaPids(int Source, int Transponder, int ServiceId, const int *CaSystemIds, int BufSize, int *Pids);
+  int GetPmtPid(int Source, int Transponder, int ServiceId);
   };
 
 int cCaDescriptorHandler::AddCaDescriptors(cCaDescriptors *CaDescriptors)
@@ -210,117 +232,384 @@ int cCaDescriptorHandler::AddCaDescriptors(cCaDescriptors *CaDescriptors)
   return CaDescriptors->Empty() ? 0 : 1;
 }
 
-int cCaDescriptorHandler::GetCaDescriptors(int Source, int Transponder, int ServiceId, const int *CaSystemIds, int BufSize, uchar *Data, int EsPid)
+void cCaDescriptorHandler::GetCaDescriptors(int Source, int Transponder, int ServiceId, const int *CaSystemIds, cDynamicBuffer &Buffer, int EsPid)
+{
+  cMutexLock MutexLock(&mutex);
+  for (cCaDescriptors *ca = First(); ca; ca = Next(ca)) {
+      if (ca->Is(Source, Transponder, ServiceId)) {
+         ca->GetCaDescriptors(CaSystemIds, Buffer, EsPid);
+         break;
+         }
+      }
+}
+
+int cCaDescriptorHandler::GetCaPids(int Source, int Transponder, int ServiceId, const int *CaSystemIds, int BufSize, int *Pids)
 {
   cMutexLock MutexLock(&mutex);
   for (cCaDescriptors *ca = First(); ca; ca = Next(ca)) {
       if (ca->Is(Source, Transponder, ServiceId))
-         return ca->GetCaDescriptors(CaSystemIds, BufSize, Data, EsPid);
+         return ca->GetCaPids(CaSystemIds, BufSize, Pids);
+      }
+  return 0;
+}
+
+int cCaDescriptorHandler::GetPmtPid(int Source, int Transponder, int ServiceId)
+{
+  cMutexLock MutexLock(&mutex);
+  for (cCaDescriptors *ca = First(); ca; ca = Next(ca)) {
+      if (ca->Is(Source, Transponder, ServiceId))
+         return ca->GetPmtPid();
       }
   return 0;
 }
 
 cCaDescriptorHandler CaDescriptorHandler;
 
-int GetCaDescriptors(int Source, int Transponder, int ServiceId, const int *CaSystemIds, int BufSize, uchar *Data, int EsPid)
+void GetCaDescriptors(int Source, int Transponder, int ServiceId, const int *CaSystemIds, cDynamicBuffer &Buffer, int EsPid)
 {
-  return CaDescriptorHandler.GetCaDescriptors(Source, Transponder, ServiceId, CaSystemIds, BufSize, Data, EsPid);
+  CaDescriptorHandler.GetCaDescriptors(Source, Transponder, ServiceId, CaSystemIds, Buffer, EsPid);
 }
 
+int GetCaPids(int Source, int Transponder, int ServiceId, const int *CaSystemIds, int BufSize, int *Pids)
+{
+  return CaDescriptorHandler.GetCaPids(Source, Transponder, ServiceId, CaSystemIds, BufSize, Pids);
+}
+
+int GetPmtPid(int Source, int Transponder, int ServiceId)
+{
+  return CaDescriptorHandler.GetPmtPid(Source, Transponder, ServiceId);
+}
+
+// --- cPmtPidEntry ----------------------------------------------------------
+
+class cPmtPidEntry : public cListObject {
+private:
+  int pid;
+  int count; // the number of SIDs currently requested from this PID
+  int state; // adding/deleting PIDs to/from the filter may only be done from within the Process() function,
+             // otherwise there could be a deadlock between cPatFilter::mutex and cSectionHandler::mutex;
+             // this member tells whether this PID needs to be added to (>0) or deleted from (<0) the filter
+  bool complete; // true if all SIDs on this PID have been received
+public:
+  cPmtPidEntry(int Pid);
+  int Pid(void) { return pid; }
+  int Count(void) { return count; }
+  int State(void) { int s = state; state = 0; return s; } // returns the current state and resets it
+  void SetState(int State) { state = State; } // 1 = add the PID, -1 = delete the PID, 0 = do nothing
+  void Inc(void) { if (++count == 1) state = 1; }
+  void Dec(void) { if (--count == 0) state = -1; }
+  int Complete(void) { return complete; }
+  void SetComplete(bool State) { complete = State; }
+  };
+
+cPmtPidEntry::cPmtPidEntry(int Pid)
+{
+  pid = Pid;
+  count = 0;
+  state = 0;
+  complete = false;
+}
+
+// --- cPmtSidEntry ----------------------------------------------------------
+
+class cPmtSidEntry : public cListObject {
+private:
+  int sid;
+  int pid;
+  cPmtPidEntry *pidEntry;
+  int version;
+  bool received;
+public:
+  cPmtSidEntry(int Sid, cPmtPidEntry *PidEntry);
+  int Sid(void) { return sid; }
+  int Pid(void) { return pid; }
+  cPmtPidEntry *PidEntry(void) { return pidEntry; }
+  int Version(void) { return version; }
+  int Received(void) { return received; }
+  void SetVersion(int Version) { version = Version; }
+  void SetReceived(bool State) { received = State; }
+  };
+
+cPmtSidEntry::cPmtSidEntry(int Sid, cPmtPidEntry *PidEntry)
+{
+  sid = Sid;
+  pid = PidEntry->Pid();
+  pidEntry = PidEntry;
+  version = -1;
+  received = false;
+}
+
+// --- cPmtSidRequest --------------------------------------------------------
+
+class cPmtSidRequest : public cListObject {
+private:
+  int sid;
+  int count; // the number of requests for this SID
+public:
+  cPmtSidRequest(int Sid) { sid = Sid; count = 1; }
+  int Sid(void) { return sid; }
+  int Count(void) { return count; }
+  void Inc(void) { count++; }
+  void Dec(void) { count--; }
+  };
+
 // --- cPatFilter ------------------------------------------------------------
 
+//#define DEBUG_PAT_PMT
+#ifdef DEBUG_PAT_PMT
+#define DBGLOG(a...) { cString s = cString::sprintf(a); fprintf(stderr, "%s\n", *s); dsyslog("%s", *s); }
+#else
+#define DBGLOG(a...) void()
+#endif
+
 cPatFilter::cPatFilter(void)
 {
-  pmtIndex = 0;
-  pmtPid = 0;
-  pmtSid = 0;
-  lastPmtScan = 0;
-  numPmtEntries = 0;
+  patVersion = -1;
+  activePmt = NULL;
+  transponder = 0;
+  source = 0;
   Set(0x00, 0x00);  // PAT
 }
 
 void cPatFilter::SetStatus(bool On)
 {
+  cMutexLock MutexLock(&mutex);
+  if (On) { // restart all requested PMT Pids
+     for (cPmtPidEntry *pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid))
+         pPid->SetState(pPid->Count() > 0);
+     if (activePmt && activePmt->Count() == 0) {
+        activePmt->SetState(1);
+        timer.Set(PMT_SCAN_TIMEOUT);
+        }
+     }
+  DBGLOG("PAT filter set status %d", On);
   cFilter::SetStatus(On);
-  pmtIndex = 0;
-  pmtPid = 0;
-  pmtSid = 0;
-  lastPmtScan = 0;
-  numPmtEntries = 0;
 }
 
-void cPatFilter::Trigger(void)
+bool cPatFilter::TransponderChanged(void)
 {
-  numPmtEntries = 0;
+  if (source != Source() || transponder != Transponder()) {
+     DBGLOG("PAT filter transponder changed from %d/%d to %d/%d", source, transponder, Source(), Transponder());
+     source = Source();
+     transponder = Transponder();
+     return true;
+     }
+  return false;
 }
 
-bool cPatFilter::PmtVersionChanged(int PmtPid, int Sid, int Version)
+void cPatFilter::Trigger(int)
 {
-  uint64_t v = Version;
-  v <<= 32;
-  uint64_t id = (PmtPid | (Sid << 16)) & 0x00000000FFFFFFFFLL;
-  for (int i = 0; i < numPmtEntries; i++) {
-      if ((pmtVersion[i] & 0x00000000FFFFFFFFLL) == id) {
-         bool Changed = (pmtVersion[i] & 0x000000FF00000000LL) != v;
-         if (Changed)
-            pmtVersion[i] = id | v;
-         return Changed;
+  cMutexLock MutexLock(&mutex);
+  DBGLOG("PAT filter trigger");
+  patVersion = -1;
+  sectionSyncer.Reset();
+}
+
+void cPatFilter::Request(int Sid)
+{
+  cMutexLock MutexLock(&mutex);
+  DBGLOG("PAT filter request SID %d", Sid);
+  for (cPmtSidRequest *sr = pmtSidRequestList.First(); sr; sr = pmtSidRequestList.Next(sr)) {
+      if (sr->Sid() == Sid) {
+         sr->Inc();
+         DBGLOG("PAT filter add SID request %d (%d)", Sid, sr->Count());
+         return;
          }
       }
-  if (numPmtEntries < MAXPMTENTRIES)
-     pmtVersion[numPmtEntries++] = id | v;
+  DBGLOG("PAT filter new SID request %d", Sid);
+  pmtSidRequestList.Add(new cPmtSidRequest(Sid));
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Sid() == Sid) {
+         cPmtPidEntry *pPid = se->PidEntry();
+         pPid->Inc();
+         DBGLOG("    PMT pid %5d  SID %5d (%d)", pPid->Pid(), se->Sid(), pPid->Count());
+         break;
+         }
+      }
+}
+
+void cPatFilter::Release(int Sid)
+{
+  cMutexLock MutexLock(&mutex);
+  DBGLOG("PAT filter release SID %d", Sid);
+  for (cPmtSidRequest *sr = pmtSidRequestList.First(); sr; sr = pmtSidRequestList.Next(sr)) {
+      if (sr->Sid() == Sid) {
+         sr->Dec();
+         DBGLOG("PAT filter del SID request %d (%d)", Sid, sr->Count());
+         if (sr->Count() == 0) {
+            pmtSidRequestList.Del(sr);
+            for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+                if (se->Sid() == Sid) {
+                   cPmtPidEntry *pPid = se->PidEntry();
+                   pPid->Dec();
+                   DBGLOG("    PMT pid %5d  SID %5d (%d)", pPid->Pid(), se->Sid(), pPid->Count());
+                   break;
+                   }
+                }
+            }
+         break;
+         }
+      }
+}
+
+int cPatFilter::NumSidRequests(int Sid)
+{
+  for (cPmtSidRequest *sr = pmtSidRequestList.First(); sr; sr = pmtSidRequestList.Next(sr)) {
+      if (sr->Sid() == Sid)
+         return sr->Count();
+      }
+  return 0;
+}
+
+bool cPatFilter::PmtPidComplete(int PmtPid)
+{
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Pid() == PmtPid && !se->Received())
+         return false;
+      }
   return true;
 }
 
+void cPatFilter::PmtPidReset(int PmtPid)
+{
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Pid() == PmtPid) {
+         se->SetReceived(false);
+         se->PidEntry()->SetComplete(false);
+         }
+      }
+}
+
+bool cPatFilter::PmtVersionChanged(int PmtPid, int Sid, int Version, bool SetNewVersion)
+{
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Sid() == Sid && se->Pid() == PmtPid) {
+         if (!se->Received()) {
+            se->SetReceived(true);
+            se->PidEntry()->SetComplete(PmtPidComplete(PmtPid));
+            }
+         if (se->Version() != Version) {
+            if (SetNewVersion)
+               se->SetVersion(Version);
+            else
+               DBGLOG("PMT %d  %5d/%5d  %2d -> %2d  %d", Transponder(), PmtPid, Sid, se->Version(), Version, NumSidRequests(Sid));
+            return true;
+            }
+         break;
+         }
+      }
+  return false;
+}
+
+void cPatFilter::SwitchToNextPmtPid(void)
+{
+  if (activePmt) {
+     if (activePmt->Count() == 0)
+        Del(activePmt->Pid(), SI::TableIdPMT);
+     for (;;) {
+         activePmt = pmtPidList.Next(activePmt);
+         if (!activePmt || activePmt->Count() == 0)
+            break;
+         }
+     if (activePmt) {
+        PmtPidReset(activePmt->Pid());
+        Add(activePmt->Pid(), SI::TableIdPMT);
+        timer.Set(PMT_SCAN_TIMEOUT);
+        }
+     }
+}
+
 void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
 {
+  cMutexLock MutexLock(&mutex);
+  if (TransponderChanged()) {
+     patVersion = -1;
+     sectionSyncer.Reset();
+     }
+  if (patVersion >= 0) {
+     for (cPmtPidEntry *pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid)) {
+         int State = pPid->State();
+         if (State > 0)
+            Add(pPid->Pid(), SI::TableIdPMT);
+         else if (State < 0)
+            Del(pPid->Pid(), SI::TableIdPMT);
+         }
+     }
+  else if (Pid != 0x00)
+     return;
   if (Pid == 0x00) {
-     if (Tid == 0x00) {
-        if (pmtPid && time(NULL) - lastPmtScan > PMT_SCAN_TIMEOUT) {
-           Del(pmtPid, 0x02);
-           pmtPid = 0;
-           pmtIndex++;
-           lastPmtScan = time(NULL);
-           }
-        if (!pmtPid) {
-           SI::PAT pat(Data, false);
-           if (!pat.CheckCRCAndParse())
-              return;
+     if (Tid == SI::TableIdPAT) {
+        SI::PAT pat(Data, false);
+        if (!pat.CheckCRCAndParse())
+           return;
+        if (sectionSyncer.Check(pat.getVersionNumber(), pat.getSectionNumber())) {
+           DBGLOG("PAT %d %d -> %d %d/%d", Transponder(), patVersion, pat.getVersionNumber(), pat.getSectionNumber(), pat.getLastSectionNumber());
+           bool NeedsSetStatus = patVersion >= 0;
+           if (pat.getVersionNumber() != patVersion) {
+              if (NeedsSetStatus)
+                 SetStatus(false); // deletes all PIDs from the filter
+              activePmt = NULL;
+              pmtSidList.Clear();
+              pmtPidList.Clear();
+              patVersion = pat.getVersionNumber();
+              }
            SI::PAT::Association assoc;
-           int Index = 0;
            for (SI::Loop::Iterator it; pat.associationLoop.getNext(assoc, it); ) {
                if (!assoc.isNITPid()) {
-                  if (Index++ >= pmtIndex && Channels.GetByServiceID(Source(), Transponder(), assoc.getServiceId())) {
-                     pmtPid = assoc.getPid();
-                     pmtSid = assoc.getServiceId();
-                     Add(pmtPid, 0x02);
+                  int PmtPid = assoc.getPid();
+                  int PmtSid = assoc.getServiceId();
+                  cPmtPidEntry *pPid = NULL;
+                  for (pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid)) {
+                      if (pPid->Pid() == PmtPid)
+                         break;
+                      }
+                  int SidRequest = NumSidRequests(PmtSid);
+                  DBGLOG("    PMT pid %5d  SID %5d%s%s", PmtPid, PmtSid, SidRequest ? " R" : "", pPid ? " S" : "");
+                  if (!pPid) { // new PMT Pid
+                     pPid = new cPmtPidEntry(PmtPid);
+                     pmtPidList.Add(pPid);
+                     }
+                  pmtSidList.Add(new cPmtSidEntry(PmtSid, pPid));
+                  if (SidRequest > 0)
+                     pPid->Inc();
+                  }
+               }
+           if (sectionSyncer.Processed(pat.getSectionNumber(), pat.getLastSectionNumber())) { // all PAT sections done
+              for (cPmtPidEntry *pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid)) {
+                  if (pPid->Count() == 0) {
+                     pPid->SetState(1);
+                     activePmt = pPid;
+                     timer.Set(PMT_SCAN_TIMEOUT);
                      break;
                      }
                   }
-               }
-           if (!pmtPid)
-              pmtIndex = 0;
+              if (NeedsSetStatus)
+                 SetStatus(true);
+              }
            }
         }
      }
-  else if (Pid == pmtPid && Tid == SI::TableIdPMT && Source() && Transponder()) {
+  else if (Tid == SI::TableIdPMT && Source() && Transponder()) {
      SI::PMT pmt(Data, false);
      if (!pmt.CheckCRCAndParse())
         return;
-     if (pmt.getServiceId() != pmtSid)
-        return; // skip broken PMT records
-     if (!PmtVersionChanged(pmtPid, pmt.getTableIdExtension(), pmt.getVersionNumber())) {
-        lastPmtScan = 0; // this triggers the next scan
+     if (!PmtVersionChanged(Pid, pmt.getTableIdExtension(), pmt.getVersionNumber(), false)) {
+        if (activePmt && activePmt->Complete())
+           SwitchToNextPmtPid();
         return;
         }
-     if (!Channels.Lock(true, 10)) {
-        numPmtEntries = 0; // to make sure we try again
+     cStateKey StateKey;
+     cChannels *Channels = cChannels::GetChannelsWrite(StateKey, 10);
+     if (!Channels)
         return;
-        }
-     cChannel *Channel = Channels.GetByServiceID(Source(), Transponder(), pmt.getServiceId());
+     PmtVersionChanged(Pid, pmt.getTableIdExtension(), pmt.getVersionNumber(), true);
+     bool ChannelsModified = false;
+     if (activePmt && activePmt->Complete())
+        SwitchToNextPmtPid();
+     cChannel *Channel = Channels->GetByServiceID(Source(), Transponder(), pmt.getServiceId());
      if (Channel) {
         SI::CaDescriptor *d;
-        cCaDescriptors *CaDescriptors = new cCaDescriptors(Channel->Source(), Channel->Transponder(), Channel->Sid());
+        cCaDescriptors *CaDescriptors = new cCaDescriptors(Channel->Source(), Channel->Transponder(), Channel->Sid(), Pid);
         // Scan the common loop:
         for (SI::Loop::Iterator it; (d = (SI::CaDescriptor*)pmt.commonDescriptors.getNext(it, SI::CaDescriptorTag)); ) {
             CaDescriptors->AddCaDescriptor(d, 0);
@@ -352,7 +641,8 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
             switch (stream.getStreamType()) {
               case 1: // STREAMTYPE_11172_VIDEO
               case 2: // STREAMTYPE_13818_VIDEO
-              case 0x1B: // MPEG4
+              case 0x1B: // H.264
+              case 0x24: // H.265
                       Vpid = esPid;
                       Ppid = pmt.getPCRPid();
                       Vtype = stream.getStreamType();
@@ -360,7 +650,7 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                       break;
               case 3: // STREAMTYPE_11172_AUDIO
               case 4: // STREAMTYPE_13818_AUDIO
-              case 0x0F: // ISO/IEC 13818-7 Audio with ADTS transport sytax
+              case 0x0F: // ISO/IEC 13818-7 Audio with ADTS transport syntax
               case 0x11: // ISO/IEC 14496-3 Audio with LATM transport syntax
                       {
                       if (NumApids < MAXAPIDS) {
@@ -456,7 +746,49 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                          }
                       }
                       break;
-              case 0x80 ... 0xFF: // STREAMTYPE_USER_PRIVATE
+              case 0x80: // STREAMTYPE_USER_PRIVATE
+                      if (Setup.StandardCompliance == STANDARD_ANSISCTE) { // DigiCipher II VIDEO (ANSI/SCTE 57)
+                         Vpid = esPid;
+                         Ppid = pmt.getPCRPid();
+                         Vtype = 0x02; // compression based upon MPEG-2
+                         ProcessCaDescriptors = true;
+                         break;
+                         }
+                      // fall through
+              case 0x81: // STREAMTYPE_USER_PRIVATE
+              case 0x87: // eac3
+                      if (Setup.StandardCompliance == STANDARD_ANSISCTE) { // ATSC A/53 AUDIO (ANSI/SCTE 57)
+                         char lang[MAXLANGCODE1] = { 0 };
+                         SI::Descriptor *d;
+                         for (SI::Loop::Iterator it; (d = stream.streamDescriptors.getNext(it)); ) {
+                             switch (d->getDescriptorTag()) {
+                               case SI::ISO639LanguageDescriptorTag: {
+                                    SI::ISO639LanguageDescriptor *ld = (SI::ISO639LanguageDescriptor *)d;
+                                    strn0cpy(lang, I18nNormalizeLanguageCode(ld->languageCode), MAXLANGCODE1);
+                                    }
+                                    break;
+                               default: ;
+                               }
+                            delete d;
+                            }
+                         if (NumDpids < MAXDPIDS) {
+                            Dpids[NumDpids] = esPid;
+                            Dtypes[NumDpids] = SI::AC3DescriptorTag;
+                            strn0cpy(DLangs[NumDpids], lang, MAXLANGCODE1);
+                            NumDpids++;
+                            }
+                         ProcessCaDescriptors = true;
+                         break;
+                         }
+                      // fall through
+              case 0x82: // STREAMTYPE_USER_PRIVATE
+                      if (Setup.StandardCompliance == STANDARD_ANSISCTE) { // STANDARD SUBTITLE (ANSI/SCTE 27)
+                         //TODO
+                         break;
+                         }
+                      // fall through
+              case 0x83 ... 0x86: // STREAMTYPE_USER_PRIVATE
+              case 0x88 ... 0xFF: // STREAMTYPE_USER_PRIVATE
                       {
                       char lang[MAXLANGCODE1] = { 0 };
                       bool IsAc3 = false;
@@ -467,17 +799,11 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                                  SI::RegistrationDescriptor *rd = (SI::RegistrationDescriptor *)d;
                                  // http://www.smpte-ra.org/mpegreg/mpegreg.html
                                  switch (rd->getFormatIdentifier()) {
-                                   case 0x44434949: // 'DCII' aka. DigiCipher II
-                                        Vpid = esPid;
-                                        Ppid = pmt.getPCRPid();
-                                        Vtype = 0x02; // DCII compression is based upon MPEG-2
-                                        ProcessCaDescriptors = true;
-                                        break;
                                    case 0x41432D33: // 'AC-3'
                                         IsAc3 = true;
                                         break;
                                    default:
-                                        //printf("Format identifier: 0x08X\n", rd->getFormatIdentifier());
+                                        //printf("Format identifier: 0x%08X (pid: %d)\n", rd->getFormatIdentifier(), esPid);
                                         break;
                                    }
                                  }
@@ -493,11 +819,11 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                          }
                       if (IsAc3) {
                          if (NumDpids < MAXDPIDS) {
-                           Dpids[NumDpids] = esPid;
-                           Dtypes[NumDpids] = SI::AC3DescriptorTag;
-                           strn0cpy(DLangs[NumDpids], lang, MAXLANGCODE1);
-                           NumDpids++;
-                           }
+                            Dpids[NumDpids] = esPid;
+                            Dtypes[NumDpids] = SI::AC3DescriptorTag;
+                            strn0cpy(DLangs[NumDpids], lang, MAXLANGCODE1);
+                            NumDpids++;
+                            }
                          ProcessCaDescriptors = true;
                          }
                       }
@@ -512,13 +838,17 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                }
             }
         if (Setup.UpdateChannels >= 2) {
-           Channel->SetPids(Vpid, Ppid, Vtype, Apids, Atypes, ALangs, Dpids, Dtypes, DLangs, Spids, SLangs, Tpid);
-           Channel->SetCaIds(CaDescriptors->CaIds());
-           Channel->SetSubtitlingDescriptors(SubtitlingTypes, CompositionPageIds, AncillaryPageIds);
+           ChannelsModified |= Channel->SetPids(Vpid, Ppid, Vtype, Apids, Atypes, ALangs, Dpids, Dtypes, DLangs, Spids, SLangs, Tpid);
+           ChannelsModified |= Channel->SetCaIds(CaDescriptors->CaIds());
+           ChannelsModified |= Channel->SetSubtitlingDescriptors(SubtitlingTypes, CompositionPageIds, AncillaryPageIds);
            }
-        Channel->SetCaDescriptors(CaDescriptorHandler.AddCaDescriptors(CaDescriptors));
+        ChannelsModified |= Channel->SetCaDescriptors(CaDescriptorHandler.AddCaDescriptors(CaDescriptors));
         }
-     lastPmtScan = 0; // this triggers the next scan
-     Channels.Unlock();
+     StateKey.Remove(ChannelsModified);
+     }
+  if (timer.TimedOut()) {
+     if (activePmt)
+        DBGLOG("PMT timeout Pid %d", activePmt->Pid());
+     SwitchToNextPmtPid();
      }
 }
