@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: player.c 2.0 2007/07/20 15:25:24 kls Exp $
+ * $Id: player.c 5.2 2024/10/13 09:47:18 kls Exp $
  */
 
 #include "player.h"
@@ -60,18 +60,27 @@ cOsdObject *cControl::GetInfo(void)
   return NULL;
 }
 
-cControl *cControl::Control(void)
+const cRecording *cControl::GetRecording(void)
 {
-  cMutexLock MutexLock(&mutex);
-  return (control && !control->hidden) ? control : NULL;
+  return NULL;
+}
+
+cString cControl::GetHeader(void)
+{
+  return "";
+}
+
+cControl *cControl::Control(cMutexLock &MutexLock, bool Hidden)
+{
+  MutexLock.Lock(&mutex);
+  return (control && (!control->hidden || Hidden)) ? control : NULL;
 }
 
 void cControl::Launch(cControl *Control)
 {
   cMutexLock MutexLock(&mutex);
-  cControl *c = control; // keeps control from pointing to uninitialized memory
+  delete control;
   control = Control;
-  delete c;
 }
 
 void cControl::Attach(void)
@@ -81,7 +90,7 @@ void cControl::Attach(void)
      if (cDevice::PrimaryDevice()->AttachPlayer(control->player))
         control->attached = true;
      else {
-        Skins.Message(mtError, tr("Channel locked (recording)!"));
+        Skins.Message(mtError, tr("Primary device has no MPEG decoder, can't attach player!"));
         Shutdown();
         }
      }
