@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: plugin.h 1.13 2006/04/17 09:18:16 kls Exp $
+ * $Id: plugin.h 5.2 2025/03/02 11:03:35 kls Exp $
  */
 
 #ifndef __PLUGIN_H
@@ -15,13 +15,17 @@
 #include "osdbase.h"
 #include "tools.h"
 
-#define VDRPLUGINCREATOR(PluginClass) extern "C" void *VDRPluginCreator(void) { return new PluginClass; }
+#define VDRPLUGINCREATOR(PluginClass) \
+  extern "C" void *VDRPluginCreator(void) { return new PluginClass; } \
+  extern "C" void VDRPluginDestroyer(PluginClass *p) { delete p; }
 
 class cPlugin {
   friend class cDll;
   friend class cPluginManager;
 private:
-  static char *configDirectory;
+  static cString configDirectory;
+  static cString cacheDirectory;
+  static cString resourceDirectory;
   const char *name;
   bool started;
   void SetName(const char *s);
@@ -39,8 +43,12 @@ public:
   virtual bool Start(void);
   virtual void Stop(void);
   virtual void Housekeeping(void);
+#ifndef MUTE_DEPRECATED_MAINTHREADHOOK
+  [[deprecated("use proper locking instead")]]
+#endif
   virtual void MainThreadHook(void);
   virtual cString Active(void);
+  virtual time_t WakeupTime(void);
 
   virtual const char *MainMenuEntry(void);
   virtual cOsdObject *MainMenuAction(void);
@@ -50,14 +58,16 @@ public:
   void SetupStore(const char *Name, const char *Value = NULL);
   void SetupStore(const char *Name, int Value);
 
-  void RegisterI18n(const tI18nPhrase * const Phrases);
-
   virtual bool Service(const char *Id, void *Data = NULL);
   virtual const char **SVDRPHelpPages(void);
   virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
 
   static void SetConfigDirectory(const char *Dir);
   static const char *ConfigDirectory(const char *PluginName = NULL);
+  static void SetCacheDirectory(const char *Dir);
+  static const char *CacheDirectory(const char *PluginName = NULL);
+  static void SetResourceDirectory(const char *Dir);
+  static const char *ResourceDirectory(const char *PluginName = NULL);
   };
 
 class cDll : public cListObject {
@@ -66,9 +76,11 @@ private:
   char *args;
   void *handle;
   cPlugin *plugin;
+  typedef void destroy_t(cPlugin *);
+  destroy_t *destroy;
 public:
   cDll(const char *FileName, const char *Args);
-  virtual ~cDll();
+  virtual ~cDll() override;
   bool Load(bool Log = false);
   cPlugin *Plugin(void) { return plugin; }
   };
@@ -93,6 +105,7 @@ public:
   void Housekeeping(void);
   void MainThreadHook(void);
   static bool Active(const char *Prompt = NULL);
+  static cPlugin *GetNextWakeupPlugin(void);
   static bool HasPlugins(void);
   static cPlugin *GetPlugin(int Index);
   static cPlugin *GetPlugin(const char *Name);
