@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 2.12 2011/02/25 14:58:31 kls Exp $
+ * $Id: tools.c 5.18 2025/06/23 09:23:05 kls Exp $
  */
 
 #include "tools.h"
@@ -18,6 +18,7 @@ extern "C" {
 #include <jpeglib.h>
 #undef boolean
 }
+#include <locale.h>
 #include <stdlib.h>
 #include <sys/time.h>
 #include <sys/vfs.h>
@@ -130,8 +131,11 @@ char *strcpyrealloc(char *dest, const char *src)
 char *strn0cpy(char *dest, const char *src, size_t n)
 {
   char *s = dest;
-  for ( ; --n && (*dest = *src) != 0; dest++, src++) ;
-  *dest = 0;
+  if (dest && n) {
+     if (src)
+        for ( ; --n && (*dest = *src) != 0; dest++, src++) ;
+     *dest = 0;
+     }
   return s;
 }
 
@@ -150,28 +154,76 @@ char *strreplace(char *s, char c1, char c2)
 
 char *strreplace(char *s, const char *s1, const char *s2)
 {
-  char *p = strstr(s, s1);
-  if (p) {
-     int of = p - s;
+  if (!s || !s1 || !*s1 || !s2 || strcmp(s1, s2) == 0)
+     return s;
+  char *q = s;
+  if (char *p = strstr(s, s1)) {
      int l  = strlen(s);
      int l1 = strlen(s1);
      int l2 = strlen(s2);
-     if (l2 > l1) {
-        if (char *NewBuffer = (char *)realloc(s, l + l2 - l1 + 1))
-           s = NewBuffer;
-        else {
-           esyslog("ERROR: out of memory");
-           return s;
+     do {
+        int of = p - s;
+        if (l2 > l1) {
+           if (char *NewBuffer = (char *)realloc(s, l + l2 - l1 + 1))
+              s = NewBuffer;
+           else {
+              esyslog("ERROR: out of memory");
+              return s;
+              }
            }
-        }
-     char *sof = s + of;
-     if (l2 != l1)
-        memmove(sof + l2, sof + l1, l - of - l1 + 1);
-     strncpy(sof, s2, l2);
+        char *sof = s + of;
+        if (l2 != l1) {
+           memmove(sof + l2, sof + l1, l - of - l1 + 1);
+           l += l2 - l1;
+           }
+        memcpy(sof, s2, l2);
+        q = sof + l2;
+        } while (p = strstr(q, s1));
      }
   return s;
 }
 
+const char *strchrn(const char *s, char c, size_t n)
+{
+  if (n == 0)
+     return s;
+  if (s) {
+     for ( ; *s; s++) {
+         if (*s == c && --n == 0)
+            return s;
+         }
+     }
+  return NULL;
+}
+
+int strcountchr(const char *s, char c)
+{
+  int n = 0;
+  if (s && c) {
+     for ( ; *s; s++) {
+         if (*s == c)
+            n++;
+         }
+     }
+  return n;
+}
+
+cString strgetbefore(const char *s, char c, int n)
+{
+  const char *p = strrchr(s, 0); // points to the terminating 0 of s
+  while (--p >= s) {
+        if (*p == c && --n == 0)
+           break;
+        }
+  return cString(s, p);
+}
+
+const char *strgetlast(const char *s, char c)
+{
+  const char *p = strrchr(s, c);
+  return p ? p + 1 : s;
+}
+
 char *stripspace(char *s)
 {
   if (s && *s) {
@@ -201,6 +253,30 @@ char *compactspace(char *s)
   return s;
 }
 
+char *compactchars(char *s, char c)
+{
+  if (s && *s && c) {
+     char *t = s;
+     char *p = s;
+     int n = 0;
+     while (*p) {
+           if (*p != c) {
+              *t++ = *p;
+              n = 0;
+              }
+           else if (t != s && n == 0) {
+              *t++ = *p;
+              n++;
+              }
+           p++;
+           }
+     if (n)
+        t--; // the last character was c
+     *t = 0;
+     }
+  return s;
+}
+
 cString strescape(const char *s, const char *chars)
 {
   char *buffer;
@@ -224,6 +300,40 @@ cString strescape(const char *s, const char *chars)
   return cString(s, t != NULL);
 }
 
+cString strgetval(const char *s, const char *name, char d)
+{
+  if (s && name) {
+     int l = strlen(name);
+     const char *t = s;
+     while (const char *p = strstr(t, name)) {
+           t = skipspace(p + l);
+           if (p == s || *(p - 1) <= ' ') {
+              if (*t == d) {
+                 t = skipspace(t + 1);
+                 const char *v = t;
+                 while (*t > ' ')
+                       t++;
+                 return cString(v, t);
+                 break;
+                 }
+              }
+           }
+     }
+  return NULL;
+}
+
+char *strshift(char *s, int n)
+{
+  if (s && n > 0) {
+     int l = strlen(s);
+     if (n < l)
+        memmove(s, s + n, l - n + 1); // we also copy the terminating 0!
+     else
+        *s = 0;
+     }
+  return s;
+}
+
 bool startswith(const char *s, const char *p)
 {
   while (*p) {
@@ -261,7 +371,7 @@ int numdigits(int n)
 
 bool isnumber(const char *s)
 {
-  if (!*s)
+  if (!s || !*s)
      return false;
   do {
      if (!isdigit(*s))
@@ -270,11 +380,73 @@ bool isnumber(const char *s)
   return true;
 }
 
+int64_t StrToNum(const char *s)
+{
+  char *t = NULL;
+  int64_t n = strtoll(s, &t, 10);
+  if (t) {
+     switch (*t) {
+       case 'T': n *= 1024;
+       case 'G': n *= 1024;
+       case 'M': n *= 1024;
+       case 'K': n *= 1024;
+       }
+     }
+  return n;
+}
+
+bool StrInArray(const char *a[], const char *s)
+{
+  if (a) {
+     while (*a) {
+           if (strcmp(*a, s) == 0)
+              return true;
+           a++;
+           }
+     }
+  return false;
+}
+
 cString AddDirectory(const char *DirName, const char *FileName)
 {
+  if (*FileName == '/')
+     FileName++;
   return cString::sprintf("%s/%s", DirName && *DirName ? DirName : ".", FileName);
 }
 
+#define DECIMAL_POINT_C '.'
+
+double atod(const char *s)
+{
+  static lconv *loc = localeconv();
+  if (*loc->decimal_point != DECIMAL_POINT_C) {
+     char buf[strlen(s) + 1];
+     char *p = buf;
+     while (*s) {
+           if (*s == DECIMAL_POINT_C)
+              *p = *loc->decimal_point;
+           else
+              *p = *s;
+           p++;
+           s++;
+           }
+     *p = 0;
+     return atof(buf);
+     }
+  else
+     return atof(s);
+}
+
+cString dtoa(double d, const char *Format)
+{
+  static lconv *loc = localeconv();
+  char buf[16];
+  snprintf(buf, sizeof(buf), Format, d);
+  if (*loc->decimal_point != DECIMAL_POINT_C)
+     strreplace(buf, *loc->decimal_point, DECIMAL_POINT_C);
+  return buf;
+}
+
 cString itoa(int n)
 {
   char buf[16];
@@ -294,7 +466,7 @@ bool EntriesOnSameFileSystem(const char *File1, const char *File2)
      }
   else
      LOG_ERROR_STR(File1);
-  return false;
+  return true; // we only return false if both files actually exist and are in different file systems!
 }
 
 int FreeDiskSpaceMB(const char *Directory, int *UsedMB)
@@ -369,37 +541,35 @@ bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks)
         if (d.Ok()) {
            struct dirent *e;
            while ((e = d.Next()) != NULL) {
-                 if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
-                    cString buffer = AddDirectory(FileName, e->d_name);
-                    if (FollowSymlinks) {
-                       struct stat st2;
-                       if (lstat(buffer, &st2) == 0) {
-                          if (S_ISLNK(st2.st_mode)) {
-                             int size = st2.st_size + 1;
-                             char *l = MALLOC(char, size);
-                             int n = readlink(buffer, l, size - 1);
-                             if (n < 0) {
-                                if (errno != EINVAL)
-                                   LOG_ERROR_STR(*buffer);
-                                }
-                             else {
-                                l[n] = 0;
-                                dsyslog("removing %s", l);
-                                if (remove(l) < 0)
-                                   LOG_ERROR_STR(l);
-                                }
-                             free(l);
+                 cString buffer = AddDirectory(FileName, e->d_name);
+                 if (FollowSymlinks) {
+                    struct stat st2;
+                    if (lstat(buffer, &st2) == 0) {
+                       if (S_ISLNK(st2.st_mode)) {
+                          int size = st2.st_size + 1;
+                          char *l = MALLOC(char, size);
+                          int n = readlink(buffer, l, size - 1);
+                          if (n < 0) {
+                             if (errno != EINVAL)
+                                LOG_ERROR_STR(*buffer);
                              }
-                          }
-                       else if (errno != ENOENT) {
-                          LOG_ERROR_STR(FileName);
-                          return false;
+                          else {
+                             l[n] = 0;
+                             dsyslog("removing %s", l);
+                             if (remove(l) < 0)
+                                LOG_ERROR_STR(l);
+                             }
+                          free(l);
                           }
                        }
-                    dsyslog("removing %s", *buffer);
-                    if (remove(buffer) < 0)
-                       LOG_ERROR_STR(*buffer);
+                    else if (errno != ENOENT) {
+                       LOG_ERROR_STR(FileName);
+                       return false;
+                       }
                     }
+                 dsyslog("removing %s", *buffer);
+                 if (remove(buffer) < 0)
+                    LOG_ERROR_STR(*buffer);
                  }
            }
         else {
@@ -420,21 +590,24 @@ bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks)
   return true;
 }
 
-bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
+bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis, const char *IgnoreFiles[])
 {
+  bool HasIgnoredFiles = false;
   cReadDir d(DirName);
   if (d.Ok()) {
      bool empty = true;
      struct dirent *e;
      while ((e = d.Next()) != NULL) {
-           if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..") && strcmp(e->d_name, "lost+found")) {
+           if (strcmp(e->d_name, "lost+found")) {
               cString buffer = AddDirectory(DirName, e->d_name);
               struct stat st;
               if (stat(buffer, &st) == 0) {
                  if (S_ISDIR(st.st_mode)) {
-                    if (!RemoveEmptyDirectories(buffer, true))
+                    if (!RemoveEmptyDirectories(buffer, true, IgnoreFiles))
                        empty = false;
                     }
+                 else if (RemoveThis && IgnoreFiles && StrInArray(IgnoreFiles, e->d_name))
+                    HasIgnoredFiles = true;
                  else
                     empty = false;
                  }
@@ -445,6 +618,19 @@ bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
               }
            }
      if (RemoveThis && empty) {
+        if (HasIgnoredFiles) {
+           while (*IgnoreFiles) {
+                 cString buffer = AddDirectory(DirName, *IgnoreFiles);
+                 if (access(buffer, F_OK) == 0) {
+                    dsyslog("removing %s", *buffer);
+                    if (remove(buffer) < 0) {
+                       LOG_ERROR_STR(*buffer);
+                       return false;
+                       }
+                    }
+                 IgnoreFiles++;
+                 }
+           }
         dsyslog("removing %s", DirName);
         if (remove(DirName) < 0) {
            LOG_ERROR_STR(DirName);
@@ -465,29 +651,27 @@ int DirSizeMB(const char *DirName)
      int size = 0;
      struct dirent *e;
      while (size >= 0 && (e = d.Next()) != NULL) {
-           if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
-              cString buffer = AddDirectory(DirName, e->d_name);
-              struct stat st;
-              if (stat(buffer, &st) == 0) {
-                 if (S_ISDIR(st.st_mode)) {
-                    int n = DirSizeMB(buffer);
-                    if (n >= 0)
-                       size += n;
-                    else
-                       size = -1;
-                    }
+           cString buffer = AddDirectory(DirName, e->d_name);
+           struct stat st;
+           if (stat(buffer, &st) == 0) {
+              if (S_ISDIR(st.st_mode)) {
+                 int n = DirSizeMB(buffer);
+                 if (n >= 0)
+                    size += n;
                  else
-                    size += st.st_size / MEGABYTE(1);
-                 }
-              else {
-                 LOG_ERROR_STR(*buffer);
-                 size = -1;
+                    size = -1;
                  }
+              else
+                 size += st.st_size / MEGABYTE(1);
+              }
+           else {
+              LOG_ERROR_STR(*buffer);
+              size = -1;
               }
            }
      return size;
      }
-  else
+  else if (errno != ENOENT)
      LOG_ERROR_STR(DirName);
   return -1;
 }
@@ -538,9 +722,17 @@ bool SpinUpDisk(const char *FileName)
   return false;
 }
 
-void TouchFile(const char *FileName)
+void TouchFile(const char *FileName, bool Create)
 {
-  if (utime(FileName, NULL) == -1 && errno != ENOENT)
+   if (Create && access(FileName, F_OK) != 0) { // the file does not exist
+      isyslog("creating file '%s'", FileName);
+      int f = open(FileName, O_WRONLY | O_CREAT, DEFFILEMODE);
+      if (f >= 0)
+         close(f);
+      else
+         LOG_ERROR_STR(FileName);
+      }
+   if (utime(FileName, NULL) == -1 && errno != ENOENT)
      LOG_ERROR_STR(FileName);
 }
 
@@ -552,11 +744,22 @@ time_t LastModifiedTime(const char *FileName)
   return 0;
 }
 
+off_t FileSize(const char *FileName)
+{
+  struct stat fs;
+  if (stat(FileName, &fs) == 0)
+     return fs.st_size;
+  return -1;
+}
+
 // --- cTimeMs ---------------------------------------------------------------
 
 cTimeMs::cTimeMs(int Ms)
 {
-  Set(Ms);
+  if (Ms >= 0)
+     Set(Ms);
+  else
+     begin = 0;
 }
 
 uint64_t cTimeMs::Now(void)
@@ -580,7 +783,7 @@ uint64_t cTimeMs::Now(void)
               esyslog("cTimeMs: clock_gettime(CLOCK_MONOTONIC) failed");
            }
         else
-           dsyslog("cTimeMs: not using monotonic clock - resolution is too bad (%ld s %ld ns)", tp.tv_sec, tp.tv_nsec);
+           dsyslog("cTimeMs: not using monotonic clock - resolution is too bad (%jd s %ld ns)", intmax_t(tp.tv_sec), tp.tv_nsec);
         }
      else
         esyslog("cTimeMs: clock_getres(CLOCK_MONOTONIC) failed");
@@ -607,12 +810,12 @@ void cTimeMs::Set(int Ms)
   begin = Now() + Ms;
 }
 
-bool cTimeMs::TimedOut(void)
+bool cTimeMs::TimedOut(void) const
 {
   return Now() >= begin;
 }
 
-uint64_t cTimeMs::Elapsed(void)
+uint64_t cTimeMs::Elapsed(void) const
 {
   return Now() - begin;
 }
@@ -714,17 +917,21 @@ char *Utf8Strn0Cpy(char *Dest, const char *Src, int n)
   if (cCharSetConv::SystemCharacterTable())
      return strn0cpy(Dest, Src, n);
   char *d = Dest;
-  while (*Src) {
-        int sl = Utf8CharLen(Src);
-        n -= sl;
-        if (n > 0) {
-           while (sl--)
-                 *d++ = *Src++;
-           }
-        else
-           break;
+  if (Dest && n > 0) {
+     if (Src) {
+        while (*Src) {
+              int sl = Utf8CharLen(Src);
+              n -= sl;
+              if (n > 0) {
+                 while (sl--)
+                       *d++ = *Src++;
+                 }
+              else
+                 break;
+              }
         }
-  *d = 0;
+     *d = 0;
+     }
   return Dest;
 }
 
@@ -792,7 +999,8 @@ cCharSetConv::cCharSetConv(const char *FromCode, const char *ToCode)
 cCharSetConv::~cCharSetConv()
 {
   free(result);
-  iconv_close(cd);
+  if (cd != (iconv_t)-1)
+     iconv_close(cd);
 }
 
 void cCharSetConv::SetSystemCharacterTable(const char *CharacterTable)
@@ -885,6 +1093,20 @@ cString::cString(const char *S, bool TakePointer)
   s = TakePointer ? (char *)S : S ? strdup(S) : NULL;
 }
 
+cString::cString(const char *S, const char *To)
+{
+  if (!S)
+     s = NULL;
+  else if (!To)
+     s = strdup(S);
+  else {
+     int l = To - S;
+     s = MALLOC(char, l + 1);
+     strncpy(s, S, l);
+     s[l] = 0;
+     }
+}
+
 cString::cString(const cString &String)
 {
   s = String.s ? strdup(String.s) : NULL;
@@ -904,13 +1126,56 @@ cString &cString::operator=(const cString &String)
   return *this;
 }
 
+cString &cString::operator=(cString &&String)
+{
+  if (this != &String) {
+     free(s);
+     s = String.s;
+     String.s = NULL;
+     }
+  return *this;
+}
+
 cString &cString::operator=(const char *String)
 {
+  if (s == String)
+    return *this;
   free(s);
   s = String ? strdup(String) : NULL;
   return *this;
 }
 
+cString &cString::Append(const char *String)
+{
+  if (String) {
+     int l1 = s ? strlen(s) : 0;
+     int l2 = strlen(String);
+     if (char *p = (char *)realloc(s, l1 + l2 + 1)) {
+        s = p;
+        strcpy(s + l1, String);
+        }
+     else
+        esyslog("ERROR: out of memory");
+     }
+  return *this;
+}
+
+cString &cString::Append(char c)
+{
+  if (c) {
+     int l1 = s ? strlen(s) : 0;
+     int l2 = 1;
+     if (char *p = (char *)realloc(s, l1 + l2 + 1)) {
+        s = p;
+        *(s + l1) = c;
+        *(s + l1 + 1) = 0;
+        }
+     else
+        esyslog("ERROR: out of memory");
+     }
+  return *this;
+}
+
 cString &cString::Truncate(int Index)
 {
   int l = strlen(s);
@@ -921,6 +1186,12 @@ cString &cString::Truncate(int Index)
   return *this;
 }
 
+cString &cString::CompactChars(char c)
+{
+  compactchars(s, c);
+  return *this;
+}
+
 cString cString::sprintf(const char *fmt, ...)
 {
   va_list ap;
@@ -934,7 +1205,7 @@ cString cString::sprintf(const char *fmt, ...)
   return cString(buffer, true);
 }
 
-cString cString::sprintf(const char *fmt, va_list &ap)
+cString cString::vsprintf(const char *fmt, va_list &ap)
 {
   char *buffer;
   if (!fmt || vasprintf(&buffer, fmt, ap) < 0) {
@@ -1018,6 +1289,15 @@ cString DateString(time_t t)
   return buf;
 }
 
+cString ShortDateString(time_t t)
+{
+  char buf[32];
+  struct tm tm_r;
+  tm *tm = localtime_r(&t, &tm_r);
+  strftime(buf, sizeof(buf), "%d.%m.%y", tm);
+  return buf;
+}
+
 cString TimeString(time_t t)
 {
   char buf[25];
@@ -1056,15 +1336,15 @@ static boolean JpegCompressEmptyOutputBuffer(j_compress_ptr cinfo)
         }
      else {
         esyslog("ERROR: out of memory");
-        return false;
+        return FALSE;
         }
      if (jcd->mem) {
         cinfo->dest->next_output_byte = jcd->mem + Used;
         cinfo->dest->free_in_buffer = jcd->size - Used;
-        return true;
+        return TRUE;
         }
      }
-  return false;
+  return FALSE;
 }
 
 static void JpegCompressTermDestination(j_compress_ptr cinfo)
@@ -1109,8 +1389,8 @@ uchar *RgbToJpeg(uchar *Mem, int Width, int Height, int &Size, int Quality)
   cinfo.in_color_space = JCS_RGB;
 
   jpeg_set_defaults(&cinfo);
-  jpeg_set_quality(&cinfo, Quality, true);
-  jpeg_start_compress(&cinfo, true);
+  jpeg_set_quality(&cinfo, Quality, TRUE);
+  jpeg_start_compress(&cinfo, TRUE);
 
   int rs = Width * 3;
   JSAMPROW rp[Height];
@@ -1124,6 +1404,20 @@ uchar *RgbToJpeg(uchar *Mem, int Width, int Height, int &Size, int Quality)
   return jcd.mem;
 }
 
+// --- GetHostName -----------------------------------------------------------
+
+const char *GetHostName(void)
+{
+  static char buffer[HOST_NAME_MAX] = "";
+  if (!*buffer) {
+     if (gethostname(buffer, sizeof(buffer)) < 0) {
+        LOG_ERROR;
+        strcpy(buffer, "vdr");
+        }
+     }
+  return buffer;
+}
+
 // --- cBase64Encoder --------------------------------------------------------
 
 const char *cBase64Encoder::b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -1176,6 +1470,47 @@ const char *cBase64Encoder::NextLine(void)
   return NULL;
 }
 
+// --- cBitStream ------------------------------------------------------------
+
+int cBitStream::GetBit(void)
+{
+  if (index >= length)
+     return 1;
+  int r = (data[index >> 3] >> (7 - (index & 7))) & 1;
+  ++index;
+  return r;
+}
+
+uint32_t cBitStream::GetBits(int n)
+{
+  uint32_t r = 0;
+  while (n--)
+        r |= GetBit() << n;
+  return r;
+}
+
+void cBitStream::ByteAlign(void)
+{
+  int n = index % 8;
+  if (n > 0)
+     SkipBits(8 - n);
+}
+
+void cBitStream::WordAlign(void)
+{
+  int n = index % 16;
+  if (n > 0)
+     SkipBits(16 - n);
+}
+
+bool cBitStream::SetLength(int Length)
+{
+  if (Length > length)
+     return false;
+  length = Length;
+  return true;
+}
+
 // --- cReadLine -------------------------------------------------------------
 
 cReadLine::cReadLine(void)
@@ -1234,9 +1569,24 @@ bool cPoller::Add(int FileHandle, bool Out)
   return false;
 }
 
+void cPoller::Del(int FileHandle, bool Out)
+{
+  if (FileHandle >= 0) {
+     for (int i = 0; i < numFileHandles; i++) {
+         if (pfd[i].fd == FileHandle && pfd[i].events == (Out ? POLLOUT : POLLIN)) {
+            if (i < numFileHandles - 1)
+               memmove(&pfd[i], &pfd[i + 1], (numFileHandles - i - 1) * sizeof(pollfd));
+            numFileHandles--;
+            }
+         }
+     }
+}
+
 bool cPoller::Poll(int TimeoutMs)
 {
   if (numFileHandles) {
+     if (TimeoutMs == 0)
+        TimeoutMs = 1; // can't let it be 0, otherwise poll() returns immediately, even if no file descriptors are ready
      if (poll(pfd, numFileHandles, TimeoutMs) != 0)
         return true; // returns true even in case of an error, to let the caller
                      // access the file and thus see the error code
@@ -1259,7 +1609,17 @@ cReadDir::~cReadDir()
 
 struct dirent *cReadDir::Next(void)
 {
-  return directory && readdir_r(directory, &u.d, &result) == 0 ? result : NULL;
+  if (directory) {
+#if !__GLIBC_PREREQ(2, 24) // readdir_r() is deprecated as of GLIBC 2.24
+     while (readdir_r(directory, &u.d, &result) == 0 && result) {
+#else
+     while ((result = readdir(directory)) != NULL) {
+#endif
+           if (strcmp(result->d_name, ".") && strcmp(result->d_name, ".."))
+              return result;
+           }
+     }
+  return NULL;
 }
 
 // --- cStringList -----------------------------------------------------------
@@ -1301,16 +1661,14 @@ bool cFileNameList::Load(const char *Directory, bool DirsOnly)
      struct dirent *e;
      if (d.Ok()) {
         while ((e = d.Next()) != NULL) {
-              if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
-                 if (DirsOnly) {
-                    struct stat ds;
-                    if (stat(AddDirectory(Directory, e->d_name), &ds) == 0) {
-                       if (!S_ISDIR(ds.st_mode))
-                          continue;
-                       }
+              if (DirsOnly) {
+                 struct stat ds;
+                 if (stat(AddDirectory(Directory, e->d_name), &ds) == 0) {
+                    if (!S_ISDIR(ds.st_mode))
+                       continue;
                     }
-                 Append(strdup(e->d_name));
                  }
+              Append(strdup(e->d_name));
               }
         Sort();
         return true;
@@ -1323,9 +1681,6 @@ bool cFileNameList::Load(const char *Directory, bool DirsOnly)
 
 // --- cFile -----------------------------------------------------------------
 
-bool cFile::files[FD_SETSIZE] = { false };
-int cFile::maxFiles = 0;
-
 cFile::cFile(void)
 {
   f = -1;
@@ -1347,58 +1702,25 @@ bool cFile::Open(const char *FileName, int Flags, mode_t Mode)
 bool cFile::Open(int FileDes)
 {
   if (FileDes >= 0) {
-     if (!IsOpen()) {
+     if (!IsOpen())
         f = FileDes;
-        if (f >= 0) {
-           if (f < FD_SETSIZE) {
-              if (f >= maxFiles)
-                 maxFiles = f + 1;
-              if (!files[f])
-                 files[f] = true;
-              else
-                 esyslog("ERROR: file descriptor %d already in files[]", f);
-              return true;
-              }
-           else
-              esyslog("ERROR: file descriptor %d is larger than FD_SETSIZE (%d)", f, FD_SETSIZE);
-           }
-        }
      else
         esyslog("ERROR: attempt to re-open file descriptor %d", FileDes);
      }
-  return false;
+  return IsOpen();
 }
 
 void cFile::Close(void)
 {
   if (f >= 0) {
      close(f);
-     files[f] = false;
      f = -1;
      }
 }
 
 bool cFile::Ready(bool Wait)
 {
-  return f >= 0 && AnyFileReady(f, Wait ? 1000 : 0);
-}
-
-bool cFile::AnyFileReady(int FileDes, int TimeoutMs)
-{
-  fd_set set;
-  FD_ZERO(&set);
-  for (int i = 0; i < maxFiles; i++) {
-      if (files[i])
-         FD_SET(i, &set);
-      }
-  if (0 <= FileDes && FileDes < FD_SETSIZE && !files[FileDes])
-     FD_SET(FileDes, &set); // in case we come in with an arbitrary descriptor
-  if (TimeoutMs == 0)
-     TimeoutMs = 10; // load gets too heavy with 0
-  struct timeval timeout;
-  timeout.tv_sec  = TimeoutMs / 1000;
-  timeout.tv_usec = (TimeoutMs % 1000) * 1000;
-  return select(FD_SETSIZE, &set, NULL, NULL, &timeout) > 0 && (FileDes < 0 || FD_ISSET(FileDes, &set));
+  return f >= 0 && FileReady(f, Wait ? 1000 : 0);
 }
 
 bool cFile::FileReady(int FileDes, int TimeoutMs)
@@ -1416,19 +1738,6 @@ bool cFile::FileReady(int FileDes, int TimeoutMs)
   return select(FD_SETSIZE, &set, NULL, NULL, (TimeoutMs >= 0) ? &timeout : NULL) > 0 && FD_ISSET(FileDes, &set);
 }
 
-bool cFile::FileReadyForWriting(int FileDes, int TimeoutMs)
-{
-  fd_set set;
-  struct timeval timeout;
-  FD_ZERO(&set);
-  FD_SET(FileDes, &set);
-  if (TimeoutMs < 100)
-     TimeoutMs = 100;
-  timeout.tv_sec  = 0;
-  timeout.tv_usec = TimeoutMs * 1000;
-  return select(FD_SETSIZE, NULL, &set, NULL, &timeout) > 0 && FD_ISSET(FileDes, &set);
-}
-
 // --- cSafeFile -------------------------------------------------------------
 
 cSafeFile::cSafeFile(const char *FileName)
@@ -1486,7 +1795,12 @@ bool cSafeFile::Close(void)
 
 // --- cUnbufferedFile -------------------------------------------------------
 
-#define USE_FADVISE
+#ifndef USE_FADVISE_READ
+#define USE_FADVISE_READ 0
+#endif
+#ifndef USE_FADVISE_WRITE
+#define USE_FADVISE_WRITE 1
+#endif
 
 #define WRITE_BUFFER KILOBYTE(800)
 
@@ -1505,7 +1819,7 @@ int cUnbufferedFile::Open(const char *FileName, int Flags, mode_t Mode)
   Close();
   fd = open(FileName, Flags, Mode);
   curpos = 0;
-#ifdef USE_FADVISE
+#if USE_FADVISE_READ || USE_FADVISE_WRITE
   begin = lastpos = ahead = 0;
   cachedstart = 0;
   cachedend = 0;
@@ -1521,7 +1835,7 @@ int cUnbufferedFile::Open(const char *FileName, int Flags, mode_t Mode)
 int cUnbufferedFile::Close(void)
 {
   if (fd >= 0) {
-#ifdef USE_FADVISE
+#if USE_FADVISE_READ || USE_FADVISE_WRITE
      if (totwritten)    // if we wrote anything make sure the data has hit the disk before
         fdatasync(fd);  // calling fadvise, as this is our last chance to un-cache it.
      posix_fadvise(fd, 0, 0, POSIX_FADV_DONTNEED);
@@ -1564,7 +1878,7 @@ off_t cUnbufferedFile::Seek(off_t Offset, int Whence)
 ssize_t cUnbufferedFile::Read(void *Data, size_t Size)
 {
   if (fd >= 0) {
-#ifdef USE_FADVISE
+#if USE_FADVISE_READ
      off_t jumped = curpos-lastpos; // nonzero means we're not at the last offset
      if ((cachedstart < cachedend) && (curpos < cachedstart || curpos > cachedend)) {
         // current position is outside the cached window -- invalidate it.
@@ -1575,9 +1889,9 @@ ssize_t cUnbufferedFile::Read(void *Data, size_t Size)
      cachedstart = min(cachedstart, curpos);
 #endif
      ssize_t bytesRead = safe_read(fd, Data, Size);
-#ifdef USE_FADVISE
      if (bytesRead > 0) {
         curpos += bytesRead;
+#if USE_FADVISE_READ
         cachedend = max(cachedend, curpos);
 
         // Read ahead:
@@ -1597,8 +1911,9 @@ ssize_t cUnbufferedFile::Read(void *Data, size_t Size)
            }
         else
            ahead = curpos; // jumped -> we really don't want any readahead, otherwise e.g. fast-rewind gets in trouble.
+#endif
         }
-
+#if USE_FADVISE_READ
      if (cachedstart < cachedend) {
         if (curpos - cachedstart > READCHUNK * 2) {
            // current position has moved forward enough, shrink tail window.
@@ -1622,7 +1937,7 @@ ssize_t cUnbufferedFile::Write(const void *Data, size_t Size)
 {
   if (fd >=0) {
      ssize_t bytesWritten = safe_write(fd, Data, Size);
-#ifdef USE_FADVISE
+#if USE_FADVISE_WRITE
      if (bytesWritten > 0) {
         begin = min(begin, curpos);
         curpos += bytesWritten;
@@ -1724,10 +2039,14 @@ bool cLockFile::Lock(int WaitSeconds)
               }
            else {
               LOG_ERROR_STR(fileName);
+              if (errno == ENOSPC) {
+                 esyslog("ERROR: can't create lock file '%s' - assuming lock anyway!", fileName);
+                 return true;
+                 }
               break;
               }
            if (WaitSeconds)
-              sleep(1);
+              cCondWait::SleepMs(1000);
            }
         } while (f < 0 && time(NULL) < Timeout);
      }
@@ -1787,12 +2106,58 @@ int cListObject::Index(void) const
   return i;
 }
 
+// --- cListGarbageCollector -------------------------------------------------
+
+#define LIST_GARBAGE_COLLECTOR_TIMEOUT 5 // seconds
+
+cListGarbageCollector ListGarbageCollector;
+
+cListGarbageCollector::cListGarbageCollector(void)
+{
+  objects = NULL;
+  lastPut = 0;
+}
+
+cListGarbageCollector::~cListGarbageCollector()
+{
+  if (objects)
+     esyslog("ERROR: ListGarbageCollector destroyed without prior Purge()!");
+}
+
+void cListGarbageCollector::Put(cListObject *Object)
+{
+  mutex.Lock();
+  Object->next = objects;
+  objects = Object;
+  lastPut = time(NULL);
+  mutex.Unlock();
+}
+
+void cListGarbageCollector::Purge(bool Force)
+{
+  mutex.Lock();
+  if (objects && (time(NULL) - lastPut > LIST_GARBAGE_COLLECTOR_TIMEOUT || Force)) {
+     // We make sure that any object stays in the garbage collector for at least
+     // LIST_GARBAGE_COLLECTOR_TIMEOUT seconds, to give objects that have pointers
+     // to them a chance to drop these references before the object is finally
+     // deleted.
+     while (cListObject *Object = objects) {
+           objects = Object->next;
+           delete Object;
+           }
+     }
+  mutex.Unlock();
+}
+
 // --- cListBase -------------------------------------------------------------
 
-cListBase::cListBase(void)
+cListBase::cListBase(const char *NeedsLocking)
+:stateLock(NeedsLocking)
 {
   objects = lastObject = NULL;
   count = 0;
+  needsLocking = NeedsLocking;
+  useGarbageCollector = needsLocking;
 }
 
 cListBase::~cListBase()
@@ -1800,6 +2165,15 @@ cListBase::~cListBase()
   Clear();
 }
 
+bool cListBase::Lock(cStateKey &StateKey, bool Write, int TimeoutMs) const
+{
+  if (needsLocking)
+     return stateLock.Lock(StateKey, Write, TimeoutMs);
+  else
+     esyslog("ERROR: cListBase::Lock() called for a list that doesn't require locking");
+  return false;
+}
+
 void cListBase::Add(cListObject *Object, cListObject *After)
 {
   if (After && After != lastObject) {
@@ -1839,8 +2213,12 @@ void cListBase::Del(cListObject *Object, bool DeleteObject)
   if (Object == lastObject)
      lastObject = Object->Prev();
   Object->Unlink();
-  if (DeleteObject)
-     delete Object;
+  if (DeleteObject) {
+     if (useGarbageCollector)
+        ListGarbageCollector.Put(Object);
+     else
+        delete Object;
+     }
   count--;
 }
 
@@ -1851,7 +2229,7 @@ void cListBase::Move(int From, int To)
 
 void cListBase::Move(cListObject *From, cListObject *To)
 {
-  if (From && To) {
+  if (From && To && From != To) {
      if (From->Index() < To->Index())
         To = To->Next();
      if (From == objects)
@@ -1884,11 +2262,30 @@ void cListBase::Clear(void)
   count = 0;
 }
 
-cListObject *cListBase::Get(int Index) const
+bool cListBase::Contains(const cListObject *Object) const
+{
+  for (const cListObject *o = objects; o; o = o->Next()) {
+      if (o == Object)
+         return true;
+      }
+  return false;
+}
+
+void cListBase::SetExplicitModify(void)
+{
+  stateLock.SetExplicitModify();
+}
+
+void cListBase::SetModified(void)
+{
+  stateLock.SetModified();
+}
+
+const cListObject *cListBase::Get(int Index) const
 {
   if (Index < 0)
      return NULL;
-  cListObject *object = objects;
+  const cListObject *object = objects;
   while (object && Index-- > 0)
         object = object->Next();
   return object;
@@ -1904,7 +2301,9 @@ static int CompareListObjects(const void *a, const void *b)
 void cListBase::Sort(void)
 {
   int n = Count();
-  cListObject *a[n];
+  cListObject **a = MALLOC(cListObject *, n);
+  if (a == NULL)
+     return;
   cListObject *object = objects;
   int i = 0;
   while (object && i < n) {
@@ -1918,13 +2317,53 @@ void cListBase::Sort(void)
       count--;
       Add(a[i]);
       }
+  free(a);
+}
+
+// --- cDynamicBuffer --------------------------------------------------------
+
+cDynamicBuffer::cDynamicBuffer(int InitialSize)
+{
+  initialSize = InitialSize;
+  buffer = NULL;
+  size = used = 0;
+}
+
+cDynamicBuffer::~cDynamicBuffer()
+{
+  free(buffer);
+}
+
+bool cDynamicBuffer::Realloc(int NewSize)
+{
+  if (size < NewSize) {
+     NewSize = max(NewSize, size ? size * 3 / 2 : initialSize); // increase size by at least 50%
+     if (uchar *NewBuffer = (uchar *)realloc(buffer, NewSize)) {
+        buffer = NewBuffer;
+        size = NewSize;
+        }
+     else {
+        esyslog("ERROR: out of memory");
+        return false;
+        }
+     }
+  return true;
+}
+
+void cDynamicBuffer::Append(const uchar *Data, int Length)
+{
+  if (Assert(used + Length)) {
+     memcpy(buffer + used, Data, Length);
+     used += Length;
+     }
 }
 
 // --- cHashBase -------------------------------------------------------------
 
-cHashBase::cHashBase(int Size)
+cHashBase::cHashBase(int Size, bool OwnObjects)
 {
   size = Size;
+  ownObjects = OwnObjects;
   hashTable = (cList<cHashObject>**)calloc(size, sizeof(cList<cHashObject>*));
 }
 
@@ -1958,6 +2397,13 @@ void cHashBase::Del(cListObject *Object, unsigned int Id)
 void cHashBase::Clear(void)
 {
   for (int i = 0; i < size; i++) {
+      if (ownObjects) {
+         cList<cHashObject> *list = hashTable[i];
+         if (list) {
+            for (cHashObject *hob = list->First(); hob; hob = list->Next(hob))
+                delete hob->object;
+            }
+         }
       delete hashTable[i];
       hashTable[i] = NULL;
       }
